// $Id: Logger.java,v 1.3 2000/03/24 14:42:16 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.distributor;

import java.util.Date;
import org.mitre.simp.util.LogListener;
import org.mitre.simp.util.LogEvent;
import org.mitre.simp.client.released.MessageAppend;

/**
 * Logging for the SIMP Distributor.  An instance of this class is used to
 * write errors and events into a text append pane.
 * @author John D. Ramsdell
 * @version March 2000 
 */
public class Logger
    implements LogListener
{
    private final boolean showDate;
    private final MessageAppend transcript;

    /**
     * Creates a logger.
     * @param transcript the text append pane for messages
     * @param showDate add dates to each line of output
     */
    public Logger(MessageAppend transcript, boolean showDate) { 
	this.transcript = transcript;
	this.showDate = showDate;
    }

    /**
     * This method implements the LogListener interface.
     */
    public void messageGenerated(LogEvent e) 
    {
	log(e.getMessage(), e.getThrowable());
    }

    public void log(String message) {
	log(message, null);
    }

    /**
     * The main reporting method.
     */
    public void log(String message, Throwable t) {
	StringBuffer sb = new StringBuffer();
	if (showDate) {
	    sb.append("[");
	    // is this the correct date format?
	    sb.append(new Date().toString());
	    sb.append("] ");
	}
	sb.append(message);
	if (t == null) 
	    transcript.addLine(sb.toString());
	else 
	    transcript.addLines(new String[] { sb.toString(), 
					       t.toString() });
    }
}
