// $Id: BuddyTracker.java,v 1.9 2000/04/14 18:31:40 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.model;

import java.util.*;
import org.mitre.simp.util.*;

/**
 * Buddy list tracker.  Resubscribes the user to all buddies
 * periodically or upon request.
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class BuddyTracker
    implements Runnable
{
    private final ClientModel cm;
    private final BuddyList bl;
    private boolean restart = true;

    BuddyTracker(ClientModel cm, BuddyList bl) {
	this.cm = cm;
	this.bl = bl;
    }

    public void run() {
	for (;;) {
	    synchronized (this) {
		if (!restart)
		    try {
			wait(ClientModel.LEASE_DURATION);
		    }
		    catch (InterruptedException ie) { }
		restart = false;
	    }
	    if (cm.isDebugging())
		cm.log("Tracking....");
	    { 
		GroupTable gt = bl.getGroupTable();
		Enumeration e = gt.getAllBuddies();
		while (e.hasMoreElements()) {
		    IMA place = (IMA)e.nextElement();
		    cm.subscribeTo(place);
		}
	    }
	    if (cm.isDebugging())
		cm.log("Tracked");
	}
    }

    /**
     * Request that a subscription request be generated for all buddies now.
     */
    synchronized void resume() {
	restart = true;
	notify();
    }
}
