// $Id: Description.java,v 1.1 2000/03/10 16:07:52 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.model;

import java.util.Vector;
import org.mitre.simp.util.XmlProperties;
import org.mitre.simp.util.Protocol;

/**
 * The presence information associated with each user includes a
 * string that can be used for any kind of description.  The
 * description object maintains a description on the client
 * and the server.  Changes to the description are transmitted to the
 * server and components of the client can be notified of the changes
 * by registering a DescriptionListener.
 * @author John D. Ramsdell and Galen B. Williamson
 * @version February 2000 
 */
public class Description
{
    private volatile String description;

    private final ClientModel cm;
    
    Description(ClientModel cm, String description) {
	this.cm = cm;
	this.description = description == null ? "" : description;
    }

    /**
     * Returns the user's current description.
     */
    public String getDescription() {
	return description;
    }

    /**
     * Sets the user's current description.
     * On success, listeners are notified.
     */
    public synchronized Answer setDescription(String desc) {
	String old = description;
	if (desc == null)
	    desc = "";
	if (desc.equals(old))
	    return new Answer(false, "No update needed");
	ProfileAnswer pa = cm.getProfile();
	if (pa.isError())
	    return pa;
	XmlProperties profile = pa.getProfile();
	profile.setProperty(Protocol.MESSAGE, desc);
	Answer answer = cm.setProfile(profile);
	if (answer.isError())
	    return answer;
	description = desc;
	DescriptionEvent e = new DescriptionEvent(this, desc);
	descriptionChanged(e);
	return new Answer(false, "Description updated");
    }

    // description listener

    private Vector descListeners = new Vector();

    public synchronized void addDescriptionListener(DescriptionListener el) {
	descListeners.add(el);
    }

    public synchronized void removeDescriptionListener(DescriptionListener el) {
	descListeners.remove(el);
    }

    private void descriptionChanged(DescriptionEvent e) {
	Vector l;
	synchronized (this) { l = (Vector)descListeners.clone(); }
	for (int i = 0; i < l.size(); i++) {
	    ((DescriptionListener)l.elementAt(i)).descriptionChanged(e);
	}
    }
}
