// $Id: Endpoint.java,v 1.13 2000/03/13 15:37:02 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.model;

import org.mitre.simp.util.*;

/**
 * The endpoint responds to requests by the server and provide a place to
 * send requests to the server.
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class Endpoint
    extends SocketPost
    implements XpListener
{
    private final ClientModel cm;

    Endpoint(ClientModel cm, XpSocket xps) {
	    super(xps);
	    this.cm = cm;
    }

    public void xpReceived(XpEvent e) {
	XmlProperties xp = e.getXp();
	String action = xp.getProperty(Protocol.ACTION);
	// Ignore responses and send them to super.
	String id = xp.getProperty(RESPONSE_KEY);
	if (id != null) {
	    super.xpReceived(e);
	    return;
	}
	// Handle commands that are not requests.
	if (Protocol.NOTE_SUBSCRIPTION.equals(action)) {
	    cm.noteSubscriber(xp);
	    return;
	}
	if (Protocol.NOTE_SUBSCRIPTION_LAPSE.equals(action)) {
	    cm.noteUnsubscriber(xp);
	    return;
	}
	// divert not handled by this model
	// 	if (Protocol.DIVERT.equals(action)) { ... }
	id = xp.getProperty(SocketPost.REQUEST_KEY);
	if (id == null) {
	    return;
	}
	if (Protocol.SEND.equals(action)) {
	    if (cm.noteMessage(xp))
		cm.reply(id, Protocol.okay("Message received"));
	    else
		cm.reply(id, Protocol.error("Message garbled"));
	    return;
	}
	if (Protocol.NOTE_CHANGE.equals(action)) {
	    if (cm.noteInfo(xp))
		cm.reply(id, Protocol.okay("Message received"));
	    else
		cm.reply(id, Protocol.error("Message garbled"));
	    return;
	}
	if (Protocol.NOTE_SUBSCRIPTION_END.equals(action)) {
	    if (cm.noteSubscriptionEnd(xp))
		cm.reply(id, Protocol.okay("Message received"));
	    else
		cm.reply(id, Protocol.error("Message garbled"));
	    return;
	}
	if (action == null)
	    cm.reply(id, Protocol.error("Unrecognized request"));
	else
	    cm.reply(id, Protocol.error("Unrecognized request: " + action));
    }

    public void closed() {
	super.closed();
	cm.closed("Connection closed");
    }
}
