// $Id: GroupTable.java,v 1.6 2000/04/14 18:31:40 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.model;

import java.util.*;
import org.mitre.simp.util.*;

/**
 * The internal representation of a buddy list.
 * It maps group names to group lists.
 * @author John D. Ramsdell
 * @version February 2000
 */
final class GroupTable
{
    // maps a string to a group list
    private final Hashtable ht;
    // maps an IMA to a set of strings
    private final Hashtable inv;

    GroupTable() {
	this.ht = new Hashtable();
	this.inv = new Hashtable();
    }

    private GroupTable(Hashtable ht, Hashtable inv) {
	this.ht = (Hashtable)ht.clone();
	this.inv = (Hashtable)inv.clone();
    }

    public Object clone() {
	return new GroupTable(ht, inv);
    }

    GroupList get(String group) {
	return (GroupList)ht.get(group);
    }

    GroupList put(String group, GroupList gl) {
	GroupList ogl = (GroupList)ht.put(group, gl);
	removeInv(group, ogl);
	if (gl != null) {
	    IMA[] members = gl.getMembers();
	    for (int i = 0; i < members.length; i++) {
		Set groups = (Set)inv.get(members[i]);
		if (groups == null) {
		    groups = new HashSet();
		    inv.put(members[i], groups);
		}
		groups.add(group);
	    }
	}
	return ogl;
    }

    GroupList remove(String group) {
	GroupList gl = (GroupList)ht.remove(group);
	removeInv(group, gl);
	return gl;
    }

    private void removeInv(String group, GroupList gl) {
	if (gl != null) {
	    IMA[] members = gl.getMembers();
	    for (int i = 0; i < members.length; i++) {
		Set groups = (Set)inv.get(members[i]);
		if (groups != null) {
		    groups.remove(group);
		    if (groups.size() == 0)
			inv.remove(members[i]);
		}
	    }
	}
    }

    Set keySet() {
	return ht.keySet();
    }

    Enumeration keys() {
	return ht.keys();
    }

    HashSet getGroups(IMA buddy) {
	return (HashSet)inv.get(buddy);
    }

    Enumeration getAllBuddies() {
	return inv.keys();
    }

    XmlProperties asXp() {
	XmlProperties xp = new XmlProperties();
	Enumeration e = keys();
	while (e.hasMoreElements()) {
	    String group = (String)e.nextElement();
	    IMA[] members = get(group).getMembers();
	    StringBuffer sb = new StringBuffer();
	    if (members.length > 0) {
		sb.append(members[0].toString());
		for (int i = 1; i < members.length; i++) {
		    sb.append(' ');
		    sb.append(members[i].toString());
		}
	    }
	    xp.setProperty(group, sb.toString());
	}
	return xp;	    
    }
}
