// $Id: Info.java,v 1.9 2000/03/08 15:31:21 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.model;

import org.mitre.simp.util.IMA;
import org.mitre.simp.util.DateTime;
import org.mitre.simp.util.Protocol;

/**
 * Holder of presence information.
 * @author John D. Ramsdell
 * @version February 2000
 */
public class Info
{  
    private final IMA regarding;
    private final String status;
    private final DateTime online;
    private final String message;

    Info(IMA regarding, String status, 
	 DateTime online, String message) {
	this.regarding = regarding;
	this.status = status;
	this.online = online;
	this.message = message;
    }

    /**
     * Return the subject of the info.
     */
    public IMA getRegarding() {
	return regarding;
    }

    /**
     * Return the online status of the subject.
     */
    public String getStatus() {
	return status;
    }

    /**
     * Returns true if the subject is online.
     */
    public boolean isOnline() {
	return Protocol.ONLINE.equals(status);
    }

    /**
     * Return the time at which the subject connected.  This is null
     * if the user is not online. 
     */
    public DateTime getOnline() {
	return online;
    }

    /**
     * Returns the subject's message.
     */
    public String getMessage() {
	return message;
    }

    /**
     * Returns true if the information in other is the same as in
     * this object.
     */
    public boolean isInfoSame(Info other) {
	return other != null
	    && other.regarding.equals(regarding)
	    && other.status.equals(status)
	    && (other.online == online 
		|| (other.online != null
		    && other.online.nearlyEquals(online)))
	    && other.message.equals(message);
    }
}
