// $Id: MessageEvent.java,v 1.4 2000/03/08 15:31:22 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.model;

import org.mitre.simp.util.*;

/**
 * Used to notify that an instant message has been received.
 * @author John D. Ramsdell
 * @version February 2000
 */
public class MessageEvent 
    extends java.util.EventObject 
{
    private final IMA to;
    private final IMA from;
    private final IMA replyTo;
    private final String body;
    private final String type;

    MessageEvent(Object source, IMA to, IMA from,
		 IMA replyTo, String type, String body) {
	super(source);
	this.to = to;
	this.from = from;
	this.replyTo = replyTo == null ? from : replyTo;
	this.type = type;
	this.body = body;
    }

    /**
     * Returns the recipient of the messsage.
     */
    public IMA getTo() {
	return to;
    }

    /**
     * Returns the originator of the message.
     */
    public IMA getFrom() {
	return from;
    }

    /**
     * Returns the address to which one should reply to this message.
     */
    public IMA getReplyTo() {
	return replyTo;
    }

    /**
     * Returns the MIME type of the message body.
     */
    public String getType() {
	return type;
    }

    /** 
     * Returns the message body.
     */
    public String getBody() {
	return body;
    }
}
