// $Id: BuddiesTree.java,v 1.27 2000/03/31 20:04:13 gwilliam Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.io.IOException;
import java.util.*;
import java.net.*;
import org.mitre.simp.util.*;

import org.mitre.simp.client.model.*;

class BuddiesTree extends JTree
    implements BuddyListListener, SimpListener, ConnectionListener
{
    protected boolean addBuddiesAlways = false;

    protected ClientModel cm;
    protected Client client;
    protected IMA me;
    protected XmlProperties profile;
    protected BuddyList buddyList;
    protected boolean injected;
    protected DefaultTreeModel model;
    final BuddyGroupNode root = new BuddyGroupNode("Should not see this node");

    // Maps group names to GroupNodes
    protected final Hashtable groupNodeTable = new Hashtable();
    // Maps group_name+IMA strings to BuddyNodes
    protected final Hashtable buddyNodeTable = new Hashtable();
    // Maps IMAs to Vectors of group names
    protected final Hashtable buddyGroupsTable = new Hashtable();

    public BuddiesTree() {
	super(new Object[] {});
	//UIManager.put("Tree.openIcon", new ImageIcon(BuddiesTree.class.getResource("/resources/images/downarrow.gif")));
	//UIManager.put("Tree.closedIcon", new ImageIcon(BuddiesTree.class.getResource("/resources/images/rightarrow.gif")));
	setRootVisible(false);
	model = new DefaultTreeModel(root, true);
	setModel(model);

	DefaultTreeSelectionModel sm = new DefaultTreeSelectionModel();
	sm.setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
	setSelectionModel(sm);
	ToolTipManager.sharedInstance().registerComponent(this);
    }

    public void init(final Client client, BuddyList list, IMA me, XmlProperties profile) {
	this.client = client;
	this.cm = client.getClientModel();
	this.me = me;
	this.profile = profile;
	this.buddyList = list;

	BuddiesTreeCellRenderer renderer = new BuddiesTreeCellRenderer();
	setCellRenderer(renderer);

	// set up a double-click handler
	addMouseListener(new MouseAdapter() {
	    public void mouseClicked(MouseEvent e) {
		int selRow = getRowForLocation(e.getX(), e.getY());
		TreePath selPath = getPathForLocation(e.getX(), e.getY());
		if(selRow != -1) {
		    if (e.getClickCount() == 1) {
			//mySingleClick(selRow, selPath);
		    }
		    else if (e.getClickCount() == 2) {
			final IMA buddy = getSelectedBuddy(selPath);
			if (buddy != null) {
			    new Thread() {
				public void run() {
				    client.createMessagePanel(buddy);
				}}.start();
			}
		    }
		} 
	    }});
	addKeyListener(new KeyAdapter() {
		public void keyReleased(KeyEvent e) {
		    int keyCode = e.getKeyCode();
		    switch (keyCode) {
		    case KeyEvent.VK_ENTER:
			break;
		    default:
			return;
		    }
		    final IMA buddy = getSelectedBuddy();
		    if (buddy != null) {
			new Thread() {
			    public void run() {
				client.createMessagePanel(buddy);
			    }}.start();
		    }
		}
	    });
	cm.addSimpListener(this);
	cm.addConnectionListener(this);
    }

    protected ClientModel getCm() {
	return cm;
    }

    // ConnectionListener methods

    private boolean connection_established = false;

    public synchronized void connectionEstablished(ConnectionEvent e) { 
	connection_established = true;
	cm.removeConnectionListener(this);
    }

    public void connectionClosed(ConnectionEvent e) { }

    protected synchronized boolean connectionEstablished() {
	return connection_established;
    }

    // BuddyListListener methods

    public void groupRenamed(GroupRenameEvent e) {
	String oldGroup = e.getOldGroup();
	String newGroup = e.getGroup();
	BuddyGroupNode bgn = getGroupNode(oldGroup);
	if (bgn != null) {
	    TreePath[] expPaths = saveExpansionPaths();

	    bgn.renameGroup(newGroup);
	    groupNodeTable.remove(oldGroup);
	    groupNodeTable.put(newGroup, bgn);
	    BuddyGroupNode parent = (BuddyGroupNode) bgn.getParent();
	    bgn.removeFromParent();
	    parent.add(bgn);
	    model.nodeStructureChanged(parent);

	    restoreExpansionPaths(expPaths);
	}
    }

    private TreePath[] saveExpansionPaths() {
	Collection groups = groupNodeTable.values();
	Vector paths = new Vector(groups.size());
	Iterator it = groups.iterator();
	while (it.hasNext()) {
	    BuddyGroupNode bgn = (BuddyGroupNode)it.next();
	    if (bgn.getChildCount() < 1) continue;
	    BuddyNode bn = (BuddyNode)bgn.getChildAt(0);
	    TreePath path = new TreePath(new Object[] { root, bgn, bn });
	    if (isExpanded(path))
		paths.add(path);
	}
	TreePath[] p = new TreePath[paths.size()];
	return (TreePath[])paths.toArray(p);
    }

    private void restoreExpansionPaths(TreePath[] paths) {
	for (int i = 0; i < paths.length; i++) {
	    expandPath(paths[i]);
	}
    }

    public void groupAdded(BuddyListEvent e) {
	addGroup(e.getGroup());
    }

    public void groupRemoved(BuddyListEvent e) {
	removeGroup(e.getGroup());
    }

    public void buddyAdded(GroupEvent e) {
	IMA buddy = e.getBuddy();
	String group = e.getGroup();
	addBuddyToGroup(buddy, group);
	makeBuddyNodeVisible(buddy, group);
    }

    public void buddyRemoved(GroupEvent e) {
	IMA buddy = e.getBuddy();
	String group = e.getGroup();
	removeBuddyFromGroup(buddy, group);
    }

    // SimpListener methods

    public synchronized void buddyInfoChanged(BuddyInfoEvent event) {
	Info info = event.getInfo();
	IMA buddy = info.getRegarding();
	Vector groups0 = (Vector)buddyGroupsTable.get(buddy);
	if (groups0 == null) return;
	Vector groups = (Vector)groups0.clone();
	Enumeration e = groups.elements();
	while (e.hasMoreElements()) {
	    String group = (String)e.nextElement();
	    BuddyGroupNode bgn = getGroupNode(group);
	    // FIXME: bug here -- bgn should never be null!!!
	    // but when new group added we get some NPEs
	    BuddyNode bn = (BuddyNode)buddyNodeTable.get(group+buddy);
	    if (info.isOnline()) {
		if (bn == null) {
		    bn = new BuddyNode(buddy, client.getClientModel());
		    buddyNodeTable.put(group+buddy, bn);
		    bgn.add(bn);
		    model.nodesWereInserted(bgn, new int[] { bgn.getIndex(bn) });
		} else if (bgn != null && bgn.getIndex(bn) == -1) {
		    bgn.add(bn);
		    model.nodesWereInserted(bgn, new int[] { bgn.getIndex(bn) });
		}
	    } else if (!addBuddiesAlways && bn != null && bgn != null) {
		int index = bgn.getIndex(bn);
		if (index != -1) {
		    bgn.remove(bn);
		    model.nodesWereRemoved(bgn, new int[] { index }, new Object[] { bn });
		} 
	    }
	}
    }

    public void messageReceived(MessageEvent e) { }
    public void infoReceived(InfoEvent e) { }
    public void subscriptionLeased(LeaseEvent e) { }
    public void subscriptionLapsed(LeaseEvent e) { }
    public void subscribed(SubscriberEvent e) { }
    public void unsubscribed(SubscriberEvent e) { }

    // group and buddy node management methods

    protected synchronized BuddyGroupNode getGroupNode(String group) {
	return (BuddyGroupNode)groupNodeTable.get(group);
    }

    protected synchronized void addBuddyToGroup(IMA buddy, String group) {
	BuddyGroupNode bgn = getGroupNode(group);

	Vector groups = (Vector)buddyGroupsTable.get(buddy);
	if (groups == null) { 
	    groups = new Vector();
	    buddyGroupsTable.put(buddy, groups);
	}
	groups.add(group);

	BuddyNode bn = new BuddyNode(buddy, client.getClientModel());
	buddyNodeTable.put(group+buddy, bn);

	boolean add = bn.isOnline() || addBuddiesAlways;
	bgn.add(bn, add);
	if (add) {
	    model.nodesWereInserted(bgn, new int[] { bgn.getIndex(bn) });
	}
    }

    protected synchronized void removeBuddyFromGroup(IMA buddy, String group) {
	BuddyGroupNode bgn = getGroupNode(group);

	Vector groups = (Vector)buddyGroupsTable.get(buddy);
	if (groups != null) {
	    groups.remove(group);
	    if (groups.size() == 0)
		buddyGroupsTable.remove(buddy);
	}

	BuddyNode bn = (BuddyNode)buddyNodeTable.remove(group+buddy);
	int index = bgn.removeBuddyTreeNode(bn);
	if (index != -1)
	    model.nodesWereRemoved(bgn, new int[] { index }, new Object[] { bn });
    }

    protected synchronized void addGroup(String group) {
	BuddyGroupNode bgn = new BuddyGroupNode(group, client.getClientModel());
	groupNodeTable.put(group, bgn);
	root.add(bgn);
	model.nodesWereInserted(root, new int[] { root.getIndex(bgn) });
    }

    protected synchronized void removeGroup(String group) {
	BuddyGroupNode bgn = (BuddyGroupNode)groupNodeTable.remove(group);
	int index = root.getIndex(bgn);
	root.remove(bgn);
	model.nodesWereRemoved(root, new int[] { index }, new Object[] { bgn });
    }

    // selection methods

    BuddyTreeNode[][] getSelectedNodes() {
	TreePath[] paths = getSelectionPaths();
	if (paths == null || paths.length < 1)
	    return null;
	Vector gnodes = new Vector(paths.length);
	Vector bnodes = new Vector(paths.length);
	for (int i = 0; i < paths.length; i++) {
	    Object o = paths[i].getLastPathComponent();
	    if (o == null)
		continue;
	    if (o instanceof BuddyGroupNode)
		gnodes.add(o);
	    else if (o instanceof BuddyNode)
		bnodes.add(o);
	}
	BuddyGroupNode[] gn = new BuddyGroupNode[gnodes.size()];
	gn = (BuddyGroupNode[])gnodes.toArray(gn);
	BuddyNode[] bn = new BuddyNode[bnodes.size()];
	bn = (BuddyNode[])bnodes.toArray(bn);
	return new BuddyTreeNode[][] { gn, bn };
    }

    String getSelectedGroupName() {
	BuddyGroupNode n = getSelectedGroupNode();
	if (n == null) return null;
	return n.getGroupName();
    }

    BuddyGroupNode getSelectedGroupNode() {
	return getSelectedGroupNode(getSelectionPath());
    }

    BuddyGroupNode getSelectedGroupNode(TreePath path) {
	if (path != null) {
	    Object o = path.getLastPathComponent();
	    if (o instanceof BuddyGroupNode) {
		BuddyGroupNode n = (BuddyGroupNode) o;
		return n;
	    }
	}
	return null;
    }

    BuddyNode getSelectedBuddyNode() {
	return getSelectedBuddyNode(getSelectionPath());
    }

    BuddyNode getSelectedBuddyNode(TreePath path) {
	if (path != null) {
	    Object o = path.getLastPathComponent();
	    if (o instanceof BuddyNode) {
		BuddyNode n = (BuddyNode) o;
		return n;
	    }
	}
	return null;
    }

    IMA getSelectedBuddy() {
	return getSelectedBuddy(getSelectionPath());
    }

    IMA getSelectedBuddy(TreePath path) {
	BuddyNode n = getSelectedBuddyNode(path);
	return (n == null) ? null : n.getBuddy();
    }

    public void makeBuddyNodeVisible(IMA buddy, String group) {
	BuddyNode node = (BuddyNode)buddyNodeTable.get(group+buddy);
	makeBuddyNodeVisible(node);
    }

    public void makeBuddyNodeVisible(final BuddyNode node) {
	final BuddyTreeNode root = this.root;
	if (node.getParent() == null) return;
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		makeVisible(new TreePath(new Object[] {
		    root, node.getParent(), node
		}));
		//invalidate();
		repaint();
	    }});
    }

    public void expandTree() {
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		// this code expands every group node
		for (Enumeration e = root.children(); 
		     e.hasMoreElements();) {
		    TreeNode n = (TreeNode)e.nextElement();
		    Object[] p;
		    if (n.getChildCount() > 0) {
			TreeNode c = n.getChildAt(0);
			p = new Object[] { root, n, c };
		    }
		    else {
			p = new Object[] { root, n };
		    }
		    makeVisible(new TreePath(p));
		}
		//invalidate();
		repaint();
	    }});
    }

    protected static ImageIcon unknownIcon;
    protected static ImageIcon buddyIcon;
    protected static ImageIcon blockedIcon;
    protected static ImageIcon expandedIcon;
    protected static ImageIcon collapsedIcon;
    protected static Dimension iconDimension = new Dimension(0,24);
    static {
	try {
	    unknownIcon = new ImageIcon (BuddiesTree.class.getResource ("/resources/images/about.gif"));
	    buddyIcon = new ImageIcon (BuddiesTree.class.getResource ("/resources/images/buddy.gif"));
	    blockedIcon = new ImageIcon (BuddiesTree.class.getResource("/resources/images/block.gif"));
	    expandedIcon = new ImageIcon (BuddiesTree.class.getResource("/resources/images/downarrow.gif"));
	    collapsedIcon = new ImageIcon (BuddiesTree.class.getResource("/resources/images/rightarrow.gif"));
	    iconDimension = MessageFrame.getMaxIconDimension(new Icon[] {
		unknownIcon, buddyIcon, blockedIcon, 
		//expandedIcon, collapsedIcon
	    });
	    iconDimension.setSize(0, iconDimension.getHeight());
	} catch (Throwable t) {
	    t.printStackTrace();
	}
    }

    class BuddiesTreeCellRenderer
	extends DefaultTreeCellRenderer
    {
	public BuddiesTreeCellRenderer() {
	}

	public Component getTreeCellRendererComponent(JTree tree,
						      Object value,
						      boolean sel,
						      boolean expanded,
						      boolean leaf,
						      int row,
						      boolean hasFocus) {

	    super.getTreeCellRendererComponent(tree, value, sel,
					       expanded, leaf, row,
					       hasFocus);
	    if (leaf && value instanceof BuddyNode) {
		BuddyNode n = (BuddyNode) value;

		BooleanAnswer answer = client.isBlocked(n.getBuddy());
		if (answer.isError())
		    setIcon(unknownIcon);
		else if (answer.getValue())
		    setIcon(blockedIcon);
		else
		    setIcon(buddyIcon);

		setToolTipText(n.getToolTipText());
	    }
	    else {
		BuddyGroupNode n = (BuddyGroupNode) value;
		setText(n.toStringWithOnlineCount());
		setToolTipText(null); //no tool tip
		/* Uncomment this to replace "folder" icons
		   if (expanded) 
		   setIcon(expandedIcon);
		   else
		   setIcon(collapsedIcon);
		*/
	    }
	    JLabel l = new JLabel(getText(), getIcon(), JLabel.LEFT);
 	    Dimension d = new Dimension(l.getPreferredSize());
	    d.setSize(d.getWidth(), iconDimension.getHeight()*1.25);
  	    setMinimumSize(d);
	    setPreferredSize(d);
  	    setMaximumSize(d);
	    return this;
	}
    }
    
}
