// $Id: BuddyGroupNode.java,v 1.11 2000/03/24 19:52:17 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.io.IOException;
import java.util.*;
import java.net.*;
import org.mitre.simp.util.*;

import org.mitre.simp.client.model.*;

/*
 * A group node is also used as the root of the buddies trees.
 */
class BuddyGroupNode
    extends BuddyTreeNode
{
    private boolean online = false;
    private String groupName;
    final Hashtable table = new Hashtable();
    final ClientModel cm;

    BuddyGroupNode(final String groupName) {
	super(groupName, groupName, true);
	this.groupName = groupName;
	this.cm = null;
    }

    BuddyGroupNode(final String groupName, ClientModel cm) {
	super(groupName, groupName, true);
	this.groupName = groupName;
	this.cm = cm;
    }

    BuddyTreeNode findChildNode(String key) {
	return (BuddyTreeNode)table.get(key);
    }

    public void add(BuddyTreeNode n) {
	add(n,true);
    }
    
    public synchronized void add(BuddyTreeNode n, boolean online) {
	table.put(n.getKey(), n);
	if (online) {
	    // We now ensure that nodes always appear in sorted order.
	    Comparable c = (Comparable) n.getUserObject();
	    int lo = 0;
	    int hi = getChildCount();
	    int i = 0;
	    while (lo < hi) {
		i = (hi - lo) / 2 + lo;
		Comparable cc = (Comparable) 
		    ((BuddyTreeNode)getChildAt(i)).getUserObject();
		int compare = c.toString().compareTo(cc.toString());
		if (compare < 0) {
		    hi = i;
		}
		else if (compare > 0) {
		    lo = i + 1;
		}
		else { lo = i; break; }
	    }
	    insert(n, lo);
	}
    }

    public int removeBuddyTreeNode(BuddyTreeNode bn) {
	table.remove(bn.getKey());
	int index = getIndex(bn);
	if (index != -1) {
	    remove(bn);
	}
	return index;
    }

    public void renameGroup(String name) {
	groupName = name;
	setUserObject(name);
    }

    public String getGroupName() { return groupName; }

    public String toStringWithOnlineCount() {
	return toString() + " (" + getChildCount() + "/" + getGroupSize() + ")";
    }

    public String toStringWithTotalCount() {
	return toString() + " (" + getGroupSize() + ")";
    }

    public String toString() {
	return groupName;
	
    }
    public int getGroupSize() { return table.size(); }

    public String[] getBuddiesAsSortedList() {
	Collection c = table.values();
	TreeSet s = new TreeSet(c);
	String[] a = new String[c.size()];
	Iterator cs = s.iterator();
	for (int i = 0; i < a.length; i++) {
	    IMA b = ((BuddyNode)cs.next()).getBuddy();
	    Info info = cm.getInfo(b);
	    String msg = (info != null)
		? info.getMessage()
		: "no info available";
	    a[i] = b+" ("+msg+")";
	}
	return a;
    }
}
