// $Id: BuddyTreeNode.java,v 1.4 2000/03/24 19:52:18 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.io.IOException;
import java.util.*;
import java.net.*;
import org.mitre.simp.util.*;

/*
 * All nodes in trees inherit from this class.
 * Like a normal node except contains a key for storing in tables and
 * provides one level of undo for set user object.
 */
abstract class BuddyTreeNode
    extends DefaultMutableTreeNode
    implements Comparable
{
    final String key;
    BuddyTreeNode(String key) {
	this(key, null,true);
    }
    BuddyTreeNode(String key, Object obj, boolean allows) {
	super(obj, allows);
	this.key = key;
    }
    String getKey() { return key; }

    protected Object savedUserObject = null;
    public void setUserObject(Object o) {
	if (!(o instanceof Comparable))
	    throw new IllegalArgumentException("Not Comparable: "+o);
	savedUserObject = getUserObject();
	super.setUserObject(o);
    }
    public Object getSavedUserObject() { 
	return savedUserObject; 
    }
    public int compareTo(Object other) {
	Comparable o = (Comparable) ((BuddyTreeNode)other).getUserObject();
	Comparable t = (Comparable) getUserObject();
	return t.compareTo(o);
    }
}

