// $Id: Client.java,v 1.62 2000/04/14 17:04:47 gwilliam Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

// Frame for the Simple Instant Messaging and Presence client

package org.mitre.simp.client.released;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.io.IOException;
import java.io.StringWriter;
import java.util.*;
import java.net.*;
import org.mitre.simp.util.*;

import org.mitre.simp.launcher.SimpLauncher;
import org.mitre.simp.launcher.Status;

import org.mitre.simp.client.model.*;

/**
 * Main Frame for the SIMP Client
 * @version December 1999
 * @author Sasha Porto Caskey
 * @author Galen B. Williamson
 */

public class Client extends JFrame
    implements Resourced,
	       ConnectionListener,
	       ACLListener,
	       SimpListener
{
    public static final String VERSION_STRING = "1.0 beta 3";

    private static final String CLIENT_PROPERTIES = "resources/client";

    private static final String MENUBAR_BUDDIES = MENUBAR+"0";
    private static final String MENUBAR_EDIT_BUDDIES = MENUBAR+"1";
    

    private static final String resPrefix = "simp.BuddyWindow.";
    private static final String tabsPrefix = "tabs";
    private static LocalResourcer res = null;

    private static int X_SIZE = -1; 
    private static int Y_SIZE = -1;

    private static final int DEFAULT_WINDOW_X = 244;
    private static final int DEFAULT_WINDOW_Y = 309;
    
    private volatile Hashtable messagePanels = new Hashtable();

    private BuddyList buddyList;
    private ClientModel cm;
    private LogFrame logFrame;

    private Font font;

    private JTabbedPane tabpane = new JTabbedPane();
    private BuddiesTree buddiesTree = new BuddiesTree();
    private EditBuddiesTree editBuddiesTree = new EditBuddiesTree();
    private JMenuBar[] menubars;
    private BuddiesTree[] buddiesTrees = {
	buddiesTree,
	editBuddiesTree
    };
    private JToolBar toolbar = null;

    boolean confirmRemove = true;

    static Resourcer resources = null;

    static {
	if (resources == null)
	    try {
		resources = new Resourcer(CLIENT_PROPERTIES);
	    }
	    catch (java.util.MissingResourceException mre) {
		System.err.println(CLIENT_PROPERTIES + ".properties not found");
		System.exit(1);
	    }
    }

    ClientModel getClientModel() { return cm; }
    BuddyList getBuddyList() { return buddyList; }

    /**
     * Creates a new instance of Client, usually called by the Login Manager
     * @param ima The IMA for user
     * @param xps The XpSocket
     * @param font A String representing the font
     * @param profile XmlProperties describing the user's profile
     * @see Login
     */
    public Client(final ClientModel cm, String font) {
	super(cm.getMe().toString() + "'s Buddies");
	/*
	try {
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} 
	catch (Exception e) { 
	}
	*/
	this.font = font != null ? Font.decode(font) : null;
	this.cm = cm;
	if (this.font != null)
	    setFont(this.font);

	buddyList = cm.getBuddyList();


	IMA me = cm.getMe();
	XmlProperties profile = cm.getProfile().getProfile();

	buddiesTree.setBounds(2,27,239,264);
	JScrollPane buddiesScroller = new JScrollPane
	    (ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
	     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	JViewport buddiesPort = buddiesScroller.getViewport();
	buddiesPort.add(buddiesTree);
	tabpane.add(buddiesScroller);

	editBuddiesTree.setBounds(2,27,239 * 4,264);
	JScrollPane editBuddiesScroller
	    = new JScrollPane(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
			      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	JViewport editBuddiesPort = editBuddiesScroller.getViewport();
	editBuddiesPort.add(editBuddiesTree);
	tabpane.add(editBuddiesScroller);

	logFrame = new LogFrame(this);

	initGUI();

	// initialize the buddies tree
	buddiesTree.init(this, buddyList, me, profile);
	buddyList.addBuddyListListener(buddiesTree);

	editBuddiesTree.init(this, cm, me, profile);
	buddyList.addBuddyListListener(editBuddiesTree);

	addWindowListener(new WindowAdapter() {
		public void windowActivated(WindowEvent event) {
		    tabpane.requestFocus();
		}
		public void windowClosing(WindowEvent event) {
		    Object object = event.getSource();
		    if (object == Client.this)
			logout(true);
		}
	    });
	addFocusListener(new FocusAdapter() {
		public void focusGained(FocusEvent event) {
		    tabpane.requestFocus();
		}
	    });
	this.show();

	cm.addConnectionListener(this);
	cm.addACLListener(this);
	cm.addSimpListener(this);

	cm.addLogListener(logFrame);
	cm.addConnectionListener(logFrame);
    
	status.showStatus("Ready");
    }

    private boolean isClosed = false;

    private volatile boolean connection_established = false;

    private boolean connectionEstablished() {
	return connection_established;
    }

    // Listener methods 

    public void connectionEstablished(ConnectionEvent e) { 
	connection_established = true;
    }

    public void connectionClosed(ConnectionEvent e) {

	// Display a dialog to inform user connections died
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		String[] options = { "Restart", "Exit" };
		int choice = JOptionPane.showOptionDialog
		    (Client.this, new String[] { 
			"Connection with server lost.",
			"Closing all session windows."
		    },
		     "Connection Closed", 
		     JOptionPane.DEFAULT_OPTION,
		     JOptionPane.WARNING_MESSAGE, null,
		     options, options[0]);
		shutdown();
		if (choice == 1)
		    System.exit(0);
		else if (!multipleLoginsAllowed())
		    SimpLauncher.restart();
	    }});

    }

    // other ACL stuff -- this should be fixed

    // It will be eliminated some day.

    private final static String ALL =
	Protocol.SEND + " " +
	Protocol.FETCH + " " +
	Protocol.SUBSCRIBE + " " +
	Protocol.UNSUBSCRIBE + " " +
	Protocol.CHANGE + " " +
	Protocol.END;

    public BooleanAnswer isBlocked(IMA who) {
	if (!connectionEstablished())
	    return new BooleanAnswer(true, "No connection established", false);
	ACLAnswer aclAnswer = cm.getACL();
	if (aclAnswer.isError()) 
	    return new BooleanAnswer(true, aclAnswer.getMessage(), false);
	ACL acl = aclAnswer.getACL();
	return new BooleanAnswer(false, aclAnswer.getMessage(), 
				 isBlocked(who, acl));
    }

    public boolean isBlocked(IMA who, ACL acl) {
	int ops = acl.getOperations(who);
	return ops == ACL.OP_NONE;
    }

    public Answer setBlocked(IMA whom, boolean blocked) {
	ACLAnswer aclAnswer = cm.getACL();
	if (aclAnswer.isError()) 
	    return aclAnswer;
	ACL acl = aclAnswer.getACL();
	int ops = acl.getOperations(whom);
	boolean permit = ops != ACL.OP_NONE;
	if (permit != blocked) {
	    // nothing to do
	    return aclAnswer; 
	}
	XmlProperties xp = (XmlProperties)acl.getXp().clone();
	String value = blocked ? "" : ALL;
	if (whom == null) 
	    xp.setProperty(Protocol.EVERYBODY, value);
	else
	    xp.setProperty(whom.toString(), value);
	ACL a = ACL.encode(xp);
	return cm.setACL(a);
    }

    public void aclChanged(ACLChangedEvent e) { 
	getSelectedBuddiesTree().repaint();
    }

    public void messageReceived(final MessageEvent e) {
	if (!e.getTo().equals(cm.getMe())) return;
	final IMA from = e.getFrom();
	if (!messagePanelExistsFor(from)) {
	    new Thread() {
		public void run() {
		    MessagePanel mp = createMessagePanel(from);
		    if (mp != null)
			// push the initial msg to the msg panel
			mp.messageReceived(e);
		}}.start();
	}
    }

    public void infoReceived(InfoEvent e) {}

    public void subscriptionLeased(LeaseEvent e) {
    }

    public void subscriptionLapsed(LeaseEvent e) {
    }

    public void subscribed(SubscriberEvent e) {
    }

    public void unsubscribed(SubscriberEvent e) {
    }

    public void buddyInfoChanged(BuddyInfoEvent e) {
    }

    /**
     * Disposes of the frame
     */
    public void dispose() {
	isClosed = true;
	shutdown(true);
    }

    public boolean messagePanelExistsFor(IMA key) {
	synchronized(messagePanels) {
	    if (messagePanels != null)
		return messagePanels.containsKey(key);
	}
	return false;
    }

    public void removeMessagePanel(IMA key) {
	synchronized(messagePanels) {
	    if (messagePanels != null)
		messagePanels.remove(key);
	}
    }

    public void addMessagePanel(IMA key, MessagePanel mp) {
	synchronized(messagePanels) {
	    if (messagePanels != null)
		messagePanels.put(key,mp);
	}
    }

    private MessagePanel getMessagePanel(IMA key) {
	synchronized(messagePanels) {
	    if (messagePanels != null)
		return (MessagePanel)messagePanels.get(key);
	}
	return null;
    }

    MessagePanel createMessagePanel() {
	IMA buddy = buddiesTree.getSelectedBuddy();
	if (buddy != null)
	    return createMessagePanel(buddy);
	return null;
    }

    MessagePanel createMessagePanel(IMA to) {
	synchronized(messagePanels) {
	    if (messagePanels != null) {
		MessagePanel mp = getMessagePanel(to);
		if (mp == null) {
		    mp = new MessagePanel(cm, to, this, font);
		    addMessagePanel(to, mp);
		}
		mp.toFront();
		return mp;
	    }
	}
	return null;
    }


    String getDefaultHost() {
	return SimpLauncher.getSimpProperties().getResourceString(SimpLauncher.SIMP_HOST_KEY);
    }

    /**
     * Notifies this component that it has been added to a container
     * This method should be called by <code>Container.add</code>, and 
     * not by user code directly.
     * Overridden here to adjust the size of the frame if needed.
     * @see java.awt.Container#removeNotify
     */
    public void addNotify()
    {
	// Record the size of the window prior to calling parents addNotify.
	Dimension size = getSize();
		
	super.addNotify();
		
	if (frameSizeAdjusted)
	    return;
	frameSizeAdjusted = true;
		
	// Adjust size of frame according to the insets and menu bar
	javax.swing.JMenuBar menuBar = getRootPane().getJMenuBar();
	int menuBarHeight = 0;
	if (menuBar != null)
	    menuBarHeight = menuBar.getPreferredSize().height;
	Insets insets = getInsets();
	setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height + menuBarHeight);
    }

    // Used by addNotify
    private boolean frameSizeAdjusted = false;

    private Status status = new Status();

    private void shutdown() { shutdown(false); }
    private void shutdown(boolean logout) {
	Hashtable mps;
	synchronized(messagePanels) {
	    mps = messagePanels;
	    messagePanels = null;
	}
	for (Enumeration e = mps.elements();
	     e.hasMoreElements();) {
	    ((MessagePanel)e.nextElement()).close();
	}
	cm.removeConnectionListener(this);
	cm.removeACLListener(this);
	cm.removeSimpListener(this);
	if (logout) cm.logout();
	setVisible(false);    // hide the Frame
	super.dispose();            // free the system resources
    }

    void logout(boolean exit) {
	String msg = exit ? "Exit" : "Logout";
	try {
	    // Beep
	    Toolkit.getDefaultToolkit().beep();
	    // Show a confirmation dialog
	    int reply = JOptionPane.showConfirmDialog
		(Client.this, 
		 "Do you really want to "+msg+"?", 
		 "Simp Client - Logout" , 
		 JOptionPane.YES_NO_OPTION, 
		 JOptionPane.QUESTION_MESSAGE);
	    // If the confirmation was affirmative, handle exiting.
	    if (reply == JOptionPane.YES_OPTION) {
		shutdown(true);
		if (exit) {
		    if (!multipleLoginsAllowed())
			System.exit(0);
		}
		else {
		    if (!multipleLoginsAllowed())
			SimpLauncher.restart();
		}
	    }
	} catch (Exception ex) {
	}

    }

    boolean multipleLoginsAllowed() {
	String key = SimpLauncher.SIMP_ALLOW_MULTIPLE_LOGINS_KEY;
	return SimpLauncher.getSimpProperties().getResourceBoolean(key);
    }

    private BuddiesTree getSelectedBuddiesTree() {
	int i = tabpane.getSelectedIndex();
	if (i < 0 || i >= buddiesTrees.length) 
	    return null;
	return buddiesTrees[i];
    }

    private JMenuItem getMenuItem(JMenuBar bar, String actionCommand) {
	return (JMenuItem)menu_items.get(bar.hashCode()+actionCommand);

    }
    private JMenuItem getActiveMenuItem(String actionCommand) {
	return getMenuItem(getJMenuBar(), actionCommand);
    }

    /**
     * Update the menus according to which tab is selected,
     * and according to the selection of buddies and/or groups.
     */
    private synchronized void updateMenus() {
	int index = tabpane.getSelectedIndex();
	if (index >= 0 && index < menubars.length)
	    setJMenuBar(menubars[index]);

	JMenuItem bii = getActiveMenuItem(infoAction);
	JMenuItem rbi = getActiveMenuItem(rembuddyAction);
	JMenuItem rgi = getActiveMenuItem(remgroupAction);
	JMenuItem bsi = getActiveMenuItem(blockAction);
	JMenuItem usi = getActiveMenuItem(unblockAction);

	Object[][] selNodes = getSelectedBuddiesTree().getSelectedNodes();

	boolean group = selNodes != null && selNodes[0].length > 0;
	boolean buddy = selNodes != null && selNodes[1].length > 0;

	boolean one_group = group && !buddy && selNodes[0].length == 1;
	boolean one_buddy = buddy && !group && selNodes[1].length == 1;

	if (bii != null) 
	    // Can only get info on one buddy or one group
	    bii.setEnabled(one_group || one_buddy);
	if (bsi != null) 
	    // Can only block if buddy(s) and/or group(s) selected
	    bsi.setEnabled(buddy || group);
	if (usi != null) 
	    // Can only unblock if buddy(s) and/or group(s) selected
	    usi.setEnabled(buddy || group);
	if (rbi != null) 
	    // Can only remove buddy(s) if only buddy(s) selected
	    rbi.setEnabled(buddy && !group);
	if (rgi != null) 
	    // Can only remove group(s) if only group(s) selected
	    rgi.setEnabled(!buddy && group);
    }

    /**
     * Initialize the GUI according to the client.properties file.
     */
    private void initGUI() {

	// initialize resources
	if (res == null)
	    res = new LocalResourcer(resPrefix, Client.resources);
	if (X_SIZE == -1)
	    X_SIZE = res.getResourceInt(WINDOW_X, DEFAULT_WINDOW_X);
	if (Y_SIZE == -1)
	    Y_SIZE = res.getResourceInt(WINDOW_Y, DEFAULT_WINDOW_Y);

	defaultActions = new Action[] {
	    // File Menu Actions
	    new LogoutAction(),
	    new ExitAction(),
	    new SendAction(),

	    // Edit Menu Actions
	    //new PreferencesAction(),
	    new NewBuddyAction(),
	    new NewGroupAction(),
	    new RemoveGroupAction(),
	    new DescriptionAction(),

	    // Buddy Menu Actions
	    new BuddyInfoAction(),
	    new BlockAction(),
	    new UnblockAction(),
	    new RemoveBuddyAction(),
	    new DefaultBlockUnblockAction(),
	    new BuddyRefreshAction(),

	    // Help Menu Actions
	    new AboutAction(),
	    new DebugAction(),
	    new ShowLogAction(),
	    new ServerInfoAction(),
	};

	// install the command table
	cmds = new HashMap();
	Action[] acts = getAugmentedActions();
	for (int i = 0; i < acts.length; i++) {
	    Action a = acts[i];
	    cmds.put(a.getValue(Action.NAME), a);
	}
	
	// set up frame
	//setTitle(cm.getMe().getUser() + "'s SIMP");
	setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

	Container cp = getContentPane();

	// set up menubars
	menubars = new JMenuBar[2];
	menubars[0] = createMenuBar(MENUBAR_BUDDIES);
	menubars[1] = createMenuBar(MENUBAR_EDIT_BUDDIES);
	if (menubars[0] != null)
	    setJMenuBar(menubars[0]);

	// set up toolbar (if any)
	toolbar = createToolBar();
	if (toolbar != null) {
	    cp.add("North", toolbar);
	    toolbar.setVisible(false);
	}

	LocalResourcer tres = new LocalResourcer(tabsPrefix+".", res);
	int[] t_bounds = tres.getResourceIntArray(boundsSuffix);
	if (t_bounds == null || t_bounds.length != 4)
	    throw new RuntimeException
		("Missing or invalid bounds in resource file.");
	tabpane.setBounds(t_bounds[0],t_bounds[1],t_bounds[2],t_bounds[3]);
	String[] tabs = res.getResourceTokenized(tabsPrefix);
	for (int i = 0; i < tabs.length; i++) {
	    String label = tres.getResourceString(tabs[i]+labelSuffix);
	    tabpane.setTitleAt(i, label);

	    int[] bounds = tres.getResourceIntArray(tabs[i]+boundsSuffix);
	    if (bounds == null || bounds.length != 4)
		throw new RuntimeException
		    ("Missing or invalid bounds in resource file.");
	    Component comp = tabpane.getComponentAt(i);
	    if (comp != null)
		comp.setBounds(bounds[0],bounds[1],bounds[2],bounds[3]);
	}

	// Create and install listeners to update menus

	TreeSelectionListener treeSelectionListener = 
	    new TreeSelectionListener() {
		public void valueChanged(TreeSelectionEvent e) { 
		    updateMenus(); 
		}};
	buddiesTree.addTreeSelectionListener(treeSelectionListener);
	editBuddiesTree.addTreeSelectionListener(treeSelectionListener);

	TreeModelListener treeModelListener = 
	    new TreeModelListener() {
		public void treeNodesChanged(TreeModelEvent e) { 
		    updateMenus(); 
		}
		public void treeNodesInserted(TreeModelEvent e) { 
		    updateMenus(); 
		}
		public void treeNodesRemoved(TreeModelEvent e) { 
		    updateMenus(); 
		}
		public void treeStructureChanged(TreeModelEvent e) { 
		    updateMenus(); 
		}};
	buddiesTree.getModel().addTreeModelListener(treeModelListener);
	editBuddiesTree.getModel().addTreeModelListener(treeModelListener);

	tabpane.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent event) { updateMenus(); }
	    });
	
	cp.add(tabpane);
	
	pack();
	setSize(X_SIZE, Y_SIZE);

	show();

	updateMenus();
    }

    // File Menu Action Names
    private static final String logoutAction = "logout";
    private static final String exitAction = "exit";
    private static final String sendAction = "send";

    // Edit Menu Action Names
    private static final String preferencesAction = "preferences";
    private static final String newbuddyAction = "newbuddy";
    private static final String rembuddyAction = "rembuddy";
    private static final String newgroupAction = "newgroup";
    private static final String remgroupAction = "remgroup";
    private static final String descriptionAction = "description";

    // Buddy Menu Action Names
    private static final String infoAction = "info";
    private static final String blockAction = "block";
    private static final String unblockAction = "unblock";
    private static final String defaultblockunblockAction = "blocka/unblocka";
    private static final String refreshAction = "refresh";

    // Help Menu Action Names
    private static final String aboutAction = "about";
    private static final String serverinfoAction = "serverinfo";
    private static final String debugAction = "debug";
    private static final String showlogAction = "showlog";

    private Action[] defaultActions;
    private Action[] augmentedActions = null;

    private BuddiesTree getBuddiesTreeForComponent(Component c) {
	if (c instanceof JMenuItem) {
	    JMenuItem mi = (JMenuItem) c;
	    JMenuBar bar = (JMenuBar)menu_bars.get(mi);
	    for (int i = 0; i < menubars.length; i++)
		if (bar == menubars[i]) {
		    return buddiesTrees[i];
		}
	}
	// won't work for buttons!!!
	return null;
    }

    private Action[] getAugmentedActions() {
	return defaultActions;
    }

    private Action getCommandAction(String act) {
	return (Action)cmds.get(act);
    }

    private Map cmds;
    private Map menu_items = Collections.synchronizedMap(new HashMap());
    private Map tool_items = Collections.synchronizedMap(new HashMap());
    private Map menu_bars = Collections.synchronizedMap(new HashMap());
    private Map tool_bars = Collections.synchronizedMap(new HashMap());

    // Automagically creates a menubar from the resource file
    private JMenuBar createMenuBar(String menubar) {
	JMenuBar bar = new JMenuBar();
	
	String[] menus = res.getResourceTokenized(menubar);
	
	for (int i = 0; i < menus.length; i++) {
	    String[] menu_items = res.getResourceTokenized(menus[i]);
	    JMenu menu = new JMenu(res.getResourceString(menus[i]+labelSuffix));
	    String menu_acc = res.getResourceString(menus[i]+accSuffix);
	    if (menu_acc != null)
		menu.setMnemonic(menu_acc.charAt(0));
	    for (int j = 0; j < menu_items.length; j++) {
		if (menu_items[j].equals("-")) {
		    menu.addSeparator();
		    continue;
		}
		String type = res.getResourceString(menu_items[j]+typeSuffix);
		if (type == null || type.equals(TYPE_CHECKBOX)) {
		    JMenuItem mi = null;
		    String label = res.getResourceString(menu_items[j]+labelSuffix);
		    if (type == null)
			mi = new JMenuItem(label);
		    else {
			boolean b = res.getResourceBoolean(menu_items[j]+defaultSuffix);
			mi = new JCheckBoxMenuItem(label, b);
		    }
		    String icon = 
			res.getResourceString(menu_items[j]+iconSuffix);
		    if (icon != null) {
			URL url = MessageFrame.class.getResource(icon);
			if (url != null)
			    mi.setIcon(new ImageIcon(url));
		    }

		    String acc = res.getResourceString(menu_items[j]+accSuffix);
		    if (acc != null)
			mi.setMnemonic(acc.charAt(0));

		    String act = res.getResourceString(menu_items[j]+actionSuffix);
		    if (act == null) act = menu_items[j];
		    mi.setActionCommand(act);
		    Action action = getCommandAction(act);
		    if (action != null) {
			mi.addActionListener(action);
		    } else {
			mi.setEnabled(false);
		    }
		    menu.add(mi);
		    this.menu_items.put(bar.hashCode()+mi.getActionCommand(), mi);
		    menu_bars.put(mi, bar);
		}
		else if (type.equals(TYPE_SWITCHABLE)) {
		    JMenuItem smi = new SwitchableMenuItem(menu_items[j], res, cmds);
		    this.menu_items.put(bar.hashCode()+smi.getActionCommand(), smi);
		    menu_bars.put(smi, bar);
		    menu.add(smi);
		}
	    }
	    bar.add(menu);
	}
	return bar;
    }

    // Automagically creates a toolbar from the resource file
    private JToolBar createToolBar() {
	String[] tools = res.getResourceTokenized(TOOLBAR);
	if (tools == null) return null;
	JToolBar bar = new JToolBar();
	for (int i = 0; i < tools.length; i++) {
	    if (tools[i].equals("-")) {
		bar.addSeparator();
		continue;
	    }
	    String type = res.getResourceString(tools[i]+typeSuffix);
	    
	    if (type == null) {
		JButton button = new JButton() {
			public float getAlignmentY() { return 0.5f; }
		    };
		button.setRequestFocusEnabled(false);
		button.setMargin(new java.awt.Insets(1,1,1,1));
		
		String icon = res.getResourceString(tools[i]+iconSuffix);
		if (icon != null) {
		    URL url = MessageFrame.class.getResource(icon);
		    if (url != null)
			button.setIcon(new ImageIcon(url));
		}
		String act = res.getResourceString(tools[i]+actionSuffix);
		if (act == null) act = tools[i];
		Action action = getCommandAction(act);
		if (action != null) {
		    button.setActionCommand(act);
		    button.addActionListener(action);
		} else {
		    button.setEnabled(false);
		}

		String tooltip = res.getResourceString(tools[i] + tipSuffix);
		if (tooltip != null)
		    button.setToolTipText(tooltip);
		bar.add(button);
		this.tool_items.put(button.getActionCommand(), button);
		tool_bars.put(button, bar);
	    }
	    else if (type.equals(TYPE_SWITCHABLE)) {
		JButton sb = new SwitchableButton(tools[i], res, cmds);
		this.tool_items.put(sb.getActionCommand(), sb);
		tool_bars.put(sb, bar);
		bar.add(sb);
	    }
	}
	return bar;
    }


    /*
     * Action classes
     */

    private class LogoutAction extends AbstractAction {
	LogoutAction() {
	    super(logoutAction);
	}
	public void actionPerformed(ActionEvent event) {
	    logout(false);
	}
    }

    private class PreferencesAction extends AbstractAction {
	PreferencesAction() {
	    super(preferencesAction);
	}
	public void actionPerformed(ActionEvent event) {
	    
	}
    }

    private class ExitAction extends AbstractAction {
	ExitAction() {
	    super(exitAction);
	}
	public void actionPerformed(ActionEvent event) {
	    logout(true);
	}
    }

    private class AboutAction extends AbstractAction {
	AboutAction() {
	    super(aboutAction);
	}
	public void actionPerformed(ActionEvent e) {
	    JOptionPane.showMessageDialog
		(Client.this, 
		 "Simp Client version "+
		 VERSION_STRING, 
		 "About the Simp Client", 
		 JOptionPane.QUESTION_MESSAGE);
	}
    }

    private class SendAction 
	extends AbstractAction 
	implements Runnable
    {
	LinkedList queue = new LinkedList();
	SendAction() {
	    super(sendAction);
	}
	public void actionPerformed(ActionEvent e) {
	    IMA bud = getSelectedBuddiesTree().getSelectedBuddy();
	    if (bud != null) 
		synchronized(queue) {
		    queue.addLast(bud);
		}
	    new Thread(this).start();
	}
	public void run() {
	    IMA bud = null;
	    synchronized(queue) {
		if (queue.size() > 0)
		    bud = (IMA)queue.removeFirst();
	    }
	    String buddy = (bud == null) ? null : bud.toString();
	    while (true) {
		JTextField buddyField = new JTextField(20);
		if (buddy != null)
		    buddyField.setText(buddy);
		Object[] message = { 
		    "Send a message to whom?",
		    buddyField 
		};
		String[] options = { "Send", "Cancel" };
		int choice = JOptionPane.showOptionDialog
		    (Client.this, message,
		     "Send...",
		     JOptionPane.DEFAULT_OPTION,
		     JOptionPane.QUESTION_MESSAGE, null,
		     options, buddyField);
		if (choice == 1 || choice == JOptionPane.CLOSED_OPTION)
		    return;
		buddy = buddyField.getText();
		IMA bima = null;
		if (buddy != null && !IMA.isIMA(buddy)) {
		    buddy = buddy + "@" + getDefaultHost();
		}
		if (IMA.isIMA(buddy)) {
		    try {
			bima = new IMA(buddy);
		    } catch(UnknownHostException uhe) {
			JOptionPane.showMessageDialog
			    (Client.this,
			     new String[] {
				 buddy,
				 "Could not be reached because",
				 "the host "+ uhe.getMessage()
				 +" is unknown.",
				 "Please try again."
			     },
			     "Send...", JOptionPane.ERROR_MESSAGE);
			continue;
		    }
		    if (bima.equals(cm.getMe())) {
			JOptionPane.showMessageDialog
			    (Client.this, "Cannot send message to self.",
			     "Error", JOptionPane.ERROR_MESSAGE);
			buddy = null;
			continue;
		    }
		    InfoAnswer answer = cm.fetch(bima);
		    if (!answer.isError() && answer.getInfo().isOnline()) {
			createMessagePanel(bima);
			return;
		    } else {
			String[] msg = 
			    answer.isError()
			    ? new String[] {
				"Could not reach "+buddy,
				"Due to the following error:",
				answer.getMessage()
			    }
			    : new String[] {
				buddy + " is not online."
			    };
			JOptionPane.showMessageDialog
			    (Client.this, msg, "Send...", 
			     JOptionPane.ERROR_MESSAGE);
			continue;
		    }
		}
		JOptionPane.showMessageDialog
		    (Client.this, 
		     new String[] {
			 "Invalid Instant Messaging Address:",
			 buddy,
			 "Please try again."
		     },
		     "Send...", JOptionPane.ERROR_MESSAGE);
	    }
	}
    }
    
    
    private class NewBuddyAction 
	extends AbstractAction implements Runnable
    {
	BuddyListAction bla = new BuddyListAction();
	NewBuddyAction() {
	    super(newbuddyAction);
	}
	public void actionPerformed(ActionEvent e) {
	    new Thread(this).start();
	}
	public void run() {
	    String buddy = null;
	    String group = null;
	    BuddyGroupNode bgn = editBuddiesTree.getSelectedGroupNode();
	    if (bgn != null) {
		group = bgn.getGroupName();
	    } else {
		BuddyNode bn = editBuddiesTree.getSelectedBuddyNode();
		if (bn != null) {
		    bgn = bn.getGroupNode();
		    if (bgn != null)
			group = bgn.getGroupName();
		}
	    }
	    while (true) {
		JTextField buddyField = new JTextField(20);
		Set availGroups = buddyList.getGroups();
		String[] groups = new String[availGroups.size()];
		groups = (String[])availGroups.toArray(groups);
		JComboBox groupBox = new JComboBox(groups);
		groupBox.setEditable(true);
		if (buddy != null)
		    buddyField.setText(buddy);
		if (group != null)
		    groupBox.setSelectedItem(group);
		Object[] message = { 
		    "Enter an Instant Messaging Address:",
		    buddyField,
		    "Select a group for the new buddy:",
		    groupBox
		};
		String[] options = { "Add", "Cancel" };
		int choice = JOptionPane.showOptionDialog
		    (Client.this, message,
		     "Add Buddy...",
		     JOptionPane.DEFAULT_OPTION,
		     JOptionPane.QUESTION_MESSAGE, null,
		     options, buddyField);
		if (choice == 1 || choice == JOptionPane.CLOSED_OPTION)
		    return;
		buddy = buddyField.getText();
		group = (String)groupBox.getSelectedItem();
		IMA bima = null;
		if (buddy != null && !IMA.isIMA(buddy)) {
		    buddy = buddy + "@" + getDefaultHost();
		}
		if (IMA.isIMA(buddy)) {
		    try {
			bima = new IMA(buddy);
		    } catch(UnknownHostException uhe) {
			JOptionPane.showMessageDialog
			    (Client.this,
			     new String[] {
				 buddy,
				 "Could not be reached because",
				 "the host "+ uhe.getMessage()
				 +" is unknown.",
				 "Please try again."
			     },
			     "Add Buddy...", JOptionPane.ERROR_MESSAGE);
			continue;
		    }
		    if (bima.equals(cm.getMe())) {
			JOptionPane.showMessageDialog
			    (Client.this, "Cannot be your own buddy.",
			     "Add Buddy...", JOptionPane.ERROR_MESSAGE);
			buddy = null;
			continue;
		    }
		    InfoAnswer answer = cm.fetch(bima);
		    if (answer.isError()) {
			choice = JOptionPane.showConfirmDialog
			    (Client.this,
			     new Object[] {
				 "Could not fetch profile information for",
				 buddy +",",
				 "due to the following error:",
				 answer.getMessage(),
				 "This user or host may not exist.",
				 "Do you wish to add this buddy anyway?",
			     },
			     "Add Buddy...",
			     JOptionPane.YES_NO_OPTION);
			if (choice != JOptionPane.YES_OPTION)
			    continue;
		    }
		    if (!buddyList.groupExists(group)) {
			message = new String[] {
			    "The group you specified does not exist.",
			    "Do you wish to create a new group called",
			    "\""+group+"\"?"
			};
			choice = JOptionPane.showConfirmDialog
			    (Client.this, message,
			     "Add Group?",
			     JOptionPane.YES_NO_OPTION);
			if (choice != JOptionPane.YES_OPTION)
			    continue;
			//Answer addAnswer = buddyList.reallyAddGroup(group);
			final String f_group = group;
			Answer addAnswer = 
			    bla.performBuddyListRequest(new Runnable() {
				    public void run() {
					buddyList.addGroup(bla, f_group);
				    }
				});
			if (addAnswer.isError()) {
			    JOptionPane.showMessageDialog
				(Client.this,
				 new String[] {
				     "Could not add the group \""+group+"\"",
				     "because of the following error:",
				     addAnswer.getMessage()
				 },
				 "Add Buddy...", 
				 JOptionPane.ERROR_MESSAGE);
			    continue;
			}
		    }

		    final IMA f_bima = bima;
		    final String f_group = group;
		    Answer addAnswer = 
			bla.performBuddyListRequest(new Runnable() {
				public void run() {
				    buddyList.addBuddy(bla, f_bima, f_group);
				}
			    });

		    if (addAnswer.isError()) {
			JOptionPane.showMessageDialog
			    (Client.this,
			     new Object[] {
				 buddy,
				 "could not be created because",
				 "of the following error:",
				 addAnswer.getMessage(),
				 "Please try again."
			     },
			     "Add Buddy...", JOptionPane.ERROR_MESSAGE);
		    }
		    else return;
		}
		JOptionPane.showMessageDialog
		    (Client.this, 
		     new String[] {
			 "Invalid Instant Messaging Address: "+buddy,
			 "Please try again."
		     },
		     "Add Buddy...", JOptionPane.ERROR_MESSAGE);
	    }
	    
	}
    }


    private class NewGroupAction 
	extends AbstractAction 
	implements Runnable
    {
	BuddyListAction bla = new BuddyListAction();
	NewGroupAction() {
	    super(newgroupAction);
	}
	public void actionPerformed(ActionEvent e) {
	    new Thread(this).start();
	}
	public void run() {
	    String group = null;
	    while (true) {
		JTextField groupField = new JTextField(20);
		if (group != null)
		    groupField.setText(group);
		Object[] message = { 
		    "Enter a name for the new group:",
		    groupField
		};
		String[] options = { "Add", "Cancel" };
		int choice = JOptionPane.showOptionDialog
		    (Client.this, message,
		     "Add Group...",
		     JOptionPane.DEFAULT_OPTION,
		     JOptionPane.QUESTION_MESSAGE, null,
		     options, groupField);
		if (choice == 1 || choice == JOptionPane.CLOSED_OPTION)
		    return;
		group = groupField.getText();
		if (buddyList.groupExists(group)) {
		    message = new String[] {
			"The group \""+group+"\" already exists.",
			"Please specify another group name, or hit cancel."
		    };
		    options = new String[] { "OK", "Cancel" };
		    choice = JOptionPane.showOptionDialog
			(Client.this, message,
			 "Group Exists",
			 JOptionPane.DEFAULT_OPTION,
			 JOptionPane.WARNING_MESSAGE,
			 null, options, options[0]);
		    if (choice == 0) {
			group = null;
			continue;
		    }
		    else return;
		}
		//Answer addAnswer = buddyList.reallyAddGroup(group);
		final String f_group = group;
		Answer addAnswer = 
		    bla.performBuddyListRequest(new Runnable() {
			    public void run() {
				buddyList.addGroup(bla, f_group);
			    }
			});
		if (addAnswer.isError()) {
		    JOptionPane.showMessageDialog
			(Client.this,
			 new String[] {
			     "Could not add the group \""+group+"\"",
			     "because of the following error:",
			     addAnswer.getMessage()
			 },
			 "Error", JOptionPane.ERROR_MESSAGE);
		    continue;
		}
		return;
	    }
	}
    }

    private class RemoveGroupAction 
	extends AbstractAction 
	implements Runnable
    {
	BuddyListAction bla = new BuddyListAction();
	RemoveGroupAction() {
	    super(remgroupAction);
	}
	public void actionPerformed(ActionEvent e) {
	    new Thread(this).start();
	}
	public void run() {
	    String group = null;
	    BuddyGroupNode bgn = editBuddiesTree.getSelectedGroupNode();
	    if (bgn == null) {
		JScrollPane  scroller = new JScrollPane
		    (ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
		     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JViewport port = scroller.getViewport();
		Set availGroups = buddyList.getGroups();
		JList list = new JList(availGroups.toArray());
		list.setSelectionMode
		    (ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		port.add(list);
		list.setVisibleRowCount(Math.min(5, availGroups.size()));
		Object[] message = new Object[] { 
		    "Please select a group or groups",
		    "to remove from the following list",
		    scroller
		};
		int choice = JOptionPane.showOptionDialog
		    (Client.this, message,
		     "Remove Groups...",
		     JOptionPane.DEFAULT_OPTION,
		     JOptionPane.QUESTION_MESSAGE, null,
		     new String[] { "Remove", "Cancel" },
		     list);
		if (choice != 0) return;
		Object[] remGroups = list.getSelectedValues();
		list = new JList(remGroups);
		list.setVisibleRowCount(Math.min(5, remGroups.length));
		scroller = new JScrollPane
		    (ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
		     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scroller.getViewport().add(list);
		choice = JOptionPane.showConfirmDialog
		    (Client.this,
		     new Object[] {
			 "Are you sure you want to remove",
			 "the following groups?",
			 scroller
		     },
		     "Remove Groups?",
		     JOptionPane.YES_NO_OPTION);
		if (choice != JOptionPane.YES_OPTION)
		    return;
		HashSet msgs = new HashSet();
		for (int i = 0; i < remGroups.length; i++) {
		    group = (String)remGroups[i];
		    //Answer answer = buddyList.reallyRemoveGroup(group);
		    final String f_group = group;
		    Answer answer = 
			bla.performBuddyListRequest(new Runnable() {
				public void run() {
				    buddyList.removeGroup(bla, f_group);
				}
			    });
		    if (answer.isError()) {
			msgs.add(group+": "+answer.getMessage());
		    }
		}
		if (msgs.size() > 0) {
		    list = new JList(msgs.toArray());
		    scroller = new JScrollPane
			(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
			 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		    scroller.getViewport().add(list);
		    JOptionPane.showMessageDialog
			(Client.this,
			 new Object[] {
			     "The following errors occurred",
			     "while trying to remove groups:",
			     scroller
			 }, 
			 "Remove Groups...", 
			 JOptionPane.ERROR_MESSAGE);
		}
		return;
	    }
	    group = bgn.getGroupName();
	    if (confirmRemove) {
		int choice = JOptionPane.showConfirmDialog
		    (Client.this,
		     new String[] {
			 "Are you sure you want to remove the group ",
			 "\""+group+"\" and all the buddies in it?"
		     },
		     "Remove Group",
		     JOptionPane.YES_NO_OPTION,
		     JOptionPane.QUESTION_MESSAGE);
		if (choice != JOptionPane.YES_OPTION) return;
	    }
	    //Answer answer = buddyList.reallyRemoveGroup(group);
	    final String f_group = group;
	    Answer answer = 
		bla.performBuddyListRequest(new Runnable() {
			public void run() {
			    buddyList.removeGroup(bla, f_group);
			}
		    });
	    if (answer.isError())
		JOptionPane.showMessageDialog
		    (Client.this,
		     new String[] {
			 "The following error occurred while trying to remove",
			 "the group \""+group+"\":",
			 answer.getMessage()
		     },
		     "Error",
		     JOptionPane.ERROR_MESSAGE);
	}
    }

    private class RemoveBuddyAction 
	extends AbstractAction 
	implements Runnable
    {
	BuddyListAction bla = new BuddyListAction();
	RemoveBuddyAction() {
	    super(rembuddyAction);
	}
	public void actionPerformed(ActionEvent e) {
	    new Thread(this).start();
	}
	public void run() {
	    BuddiesTree buddiesTree = getSelectedBuddiesTree();
	    BuddyNode bn = buddiesTree.getSelectedBuddyNode();
	    if (bn == null) return;
	    IMA buddy = bn.getBuddy();
	    BuddyGroupNode bgn = (BuddyGroupNode)bn.getParent();
	    if (buddy == null || bgn == null) 
		throw new RuntimeException("BuddyNode without parent or IMA");
	    String group = bgn.getGroupName();
	    if (confirmRemove) {
		int choice = JOptionPane.showConfirmDialog
		    (Client.this,
		     new String[] {
			 "Do you really want to remove "+buddy,
			 "from the group "+group+"?"
		     },
		     "Remove Buddy...",
		     JOptionPane.YES_NO_OPTION,
		     JOptionPane.QUESTION_MESSAGE);
		if (choice != JOptionPane.YES_OPTION) return;
	    }
	    //Answer answer = buddyList.reallyRemoveBuddy(buddy, group);
	    final String f_group = group;
	    final IMA f_buddy = buddy;
	    Answer answer = 
		bla.performBuddyListRequest(new Runnable() {
			public void run() {
			    buddyList.removeBuddy(bla, f_buddy, f_group);
			}
		    });
	    if (answer.isError())
		JOptionPane.showMessageDialog
		    (Client.this,
		     new String[] {
			 "The following error occurred while trying to remove",
			 buddy+" from "+group+":",
			 answer.getMessage()
		     },
		     "Error",
		     JOptionPane.ERROR_MESSAGE);
	}
    }

    private class BuddyInfoAction extends AbstractAction {
	BuddyInfoAction() {
	    super(infoAction);
	}
	public void actionPerformed(final ActionEvent e) {
	    new Thread() {
		    public void run() {
			performAction(e);
		    }}.start();
	}
	private void performAction(ActionEvent e) {
	    Component c = (Component)e.getSource();
	    BuddiesTree tree = getBuddiesTreeForComponent(c);
	    if (tree == null) throw new RuntimeException("info");
	    
	    Object info = null;
	    String title = "Buddy Info";

	    //BuddiesTree buddiesTree = getSelectedBuddiesTree();
	    IMA ima = tree.getSelectedBuddy();
	    if (ima != null) {
		Info buddy = cm.getInfo(ima);
		if (buddy == null) {
		    JOptionPane.showMessageDialog
			(Client.this,
			 new String[] {
			     "No information is available for",
			     ima.toString()
			 }, title, JOptionPane.ERROR_MESSAGE);
		    return;
		}
		DateTime online = buddy.getOnline();
		String since = online != null
		    ? "Online Since: "+online.getFormattedDate()
		    : "Online time unknown";
		info = new String[] {
		    "IM Address: "+buddy.getRegarding(),
		    "Description: "+buddy.getMessage(),
		    "Status: "+buddy.getStatus(),
		    since
		};
	    }
	    else {
		BuddyGroupNode bgn = buddiesTree.getSelectedGroupNode();
		if (bgn == null) return;
	        String[] buds = bgn.getBuddiesAsSortedList();
		JScrollPane scroller
		    = new JScrollPane(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		JViewport port = scroller.getViewport();
		JList list = new JList(buds);
		list.setVisibleRowCount(5);
		port.add(list);
		info = scroller;
		title = "Buddies in the \""+bgn.getGroupName()+"\" group:";
	    }
	    if (info == null) return;
	    JOptionPane.showMessageDialog
		(Client.this, info,
		 title, JOptionPane.INFORMATION_MESSAGE);
	}
    }

    private class DefaultBlockUnblockAction extends SwitchableAction 
	implements ACLListener
    {
	DefaultBlockUnblockAction() {
	    super(defaultblockunblockAction);
	    cm.addACLListener(this);
	    setState();
	}
	public void add(Switchable b) {
	    super.add(b);
	    setState(state);
	}
	public void aclChanged(ACLChangedEvent e) {
	    setState(isBlocked(null, e.getACL()));
	}
	public void setState() {
	    BooleanAnswer answer = isBlocked(null);
	    setState(!answer.isError() && answer.getValue());
	}	
	public void actionPerformed(final ActionEvent e) {
	    new Thread() {
		    public void run() {
			react(e);
		    }}.start();
	}
	private synchronized void react(final ActionEvent e) {
	    String msg = null;
	    BooleanAnswer ba = isBlocked(null);
	    if (!ba.isError()) {
		boolean blocked = ba.getValue();
		Answer answer = setBlocked(null, !blocked);
  		if (answer.isError()) msg = answer.getMessage();
  		else return;
	    } else {
		msg = ba.getMessage();
	    }
	    if (msg != null) {
		final String m = msg;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    JOptionPane.showMessageDialog
				(Client.this, 
				 new String[] {
				     "Could not change default block setting:",
				     m, 
				 },
				 "Error",
				 JOptionPane.ERROR_MESSAGE);
			}});
	    }
	}
    }

    private abstract class BlockUnblockAction extends AbstractAction {
	BlockUnblockAction(String n) {
	    super(n);
	}

	protected void performAction(ActionEvent e, boolean state) {
	    Component c = (Component)e.getSource();
	    BuddiesTree tree = getBuddiesTreeForComponent(c);
	    if (tree == null) return;

	    Vector msgs = new Vector();
	    BuddyTreeNode[][] selNodes = tree.getSelectedNodes();
	    if (selNodes != null) {
		BuddyGroupNode[] gnodes = (BuddyGroupNode[])selNodes[0];
		BuddyNode[] bnodes = (BuddyNode[])selNodes[1];
		for (int i = 0; i < gnodes.length; i++) {
		    String group = gnodes[i].getGroupName();
		    Vector v = blockUnblockGroup(group, state);
		    if (v != null) {
			msgs.addAll(v);
		    }
		}
		for (int i = 0; i < bnodes.length; i++) {
		    if (bnodes[i] == null) {
			new RuntimeException(i+","+bnodes.length).printStackTrace();
			continue;
		    }
		    IMA buddy = bnodes[i].getBuddy();
		    String m = blockUnblock(buddy, state);
		    if (m != null) 
			msgs.add(m);
		}
	    }
	    if (msgs.size() > 0) {
		String[] msg = new String[msgs.size()];
		msg = (String[])msgs.toArray(msg);
		final String[] m = msg;
		final String title = "Error while " +
		    ((state) ? "Blocking" : "Unblocking");
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    JOptionPane.showMessageDialog
				(Client.this, m, title,
				 JOptionPane.ERROR_MESSAGE);
			}});
	    }
	}

	private Vector blockUnblockGroup(String group, boolean state) {
	    String[] msg = null;
	    IMA[] imas = buddyList.getBuddies(group);
	    Vector msgs = new Vector();
	    for (int i = 0; i < imas.length; i++) {
		String m = blockUnblock(imas[i], state);
		if (m != null)
		    msgs.add(m);
	    }
	    if (msgs.size() > 0)
		return msgs;
	    return null;
	}

	private String blockUnblock(IMA buddy, boolean state) {
	    BooleanAnswer ba = isBlocked(buddy);
	    if (!ba.isError()) {
		if (ba.getValue() != state) {
		    Answer answer = setBlocked(buddy, state);
		    if (answer.isError() || cm.isDebugging()) 
			return buddy+": "+answer.getMessage();
		    else 
			return null;
		} 
		else return null;
	    } else {
		return ba.getMessage();
	    }
	}
	
    }

    private class BlockAction extends BlockUnblockAction {
	BlockAction() {
	    super(blockAction);
	}
	public void actionPerformed(final ActionEvent e) {
	    new Thread() {
		    public void run() {
			performAction(e, true);
		    }}.start();
	}
    }

    private class UnblockAction extends BlockUnblockAction {
	UnblockAction() {
	    super(unblockAction);
	}
	public void actionPerformed(final ActionEvent e) {
	    new Thread() {
		    public void run() {
			performAction(e, false);
		    }}.start();
	}
    }

    private class DebugAction extends AbstractAction {
	DebugAction() {
	    super(debugAction);
	}
	public void actionPerformed(ActionEvent e) {
	    boolean debug = cm.isDebugging();
	    cm.setDebugging(!debug);
	    for (int i = 0; i < menubars.length; i++) {
		JCheckBoxMenuItem mi = (JCheckBoxMenuItem)
		    getMenuItem(menubars[i], debugAction);
		mi.setState(cm.isDebugging());
	    }
	}
    }

    private class ShowLogAction extends AbstractAction {
	ShowLogAction() {
	    super(showlogAction);
	}
	public void actionPerformed(ActionEvent e) {
	    JCheckBoxMenuItem mi = (JCheckBoxMenuItem)e.getSource();
	    showLog(mi.getState());
	}
    }

    void showLog(boolean state) {
	logFrame.setVisible(state);
	for (int i = 0; i < menubars.length; i++) {
	    JCheckBoxMenuItem mi = (JCheckBoxMenuItem)
		getMenuItem(menubars[i], showlogAction);
	    mi.setState(state);
	}
    }

    private class ServerInfoAction extends AbstractAction {
	ServerInfoAction() {
	    super(serverinfoAction);
	}
	public void actionPerformed(ActionEvent e) {
	    Answer info = cm.inquire(cm.getMe());
	    if (!info.isError()) {
		JOptionPane.showMessageDialog
		    (Client.this,
		     info.getMessage(),
		     "Server Info for "+cm.getMe().getHost(),
		     JOptionPane.INFORMATION_MESSAGE);
	    }
	    else {
		JOptionPane.showMessageDialog
		    (Client.this,
		     info.getMessage(),
		     "Server Info",
		     JOptionPane.ERROR_MESSAGE);
	    }
	}
    }

    private class BuddyRefreshAction extends AbstractAction {
	BuddyRefreshAction() {
	    super(refreshAction);
	}
	public void actionPerformed(ActionEvent e) {
	    buddyList.resumeTracker();
	}
    }

    private class DescriptionAction extends AbstractAction {
	DescriptionAction() {
	    super(descriptionAction);
	}
	public void actionPerformed(ActionEvent e) {
	    Description desc = cm.getDescription();
	    JTextField descField = new JTextField(desc.getDescription(), 20);
	    while (true) {
		int choice = JOptionPane.showOptionDialog
		    (Client.this,
		     new Object[] {
			 "Edit your description message:",
			 descField,
		     },
		     "Edit Description...",
		     JOptionPane.DEFAULT_OPTION,
		     JOptionPane.QUESTION_MESSAGE, null,
		     new String[] { "Change", "Cancel" },
		     descField);
		if (choice != 0) return;
		String newDesc = descField.getText();
		Answer answer = desc.setDescription(newDesc);
		JOptionPane.showMessageDialog
		    (Client.this,
		     answer.getMessage(),
		     "Edit Description Result",
		     (answer.isError()
		      ? JOptionPane.ERROR_MESSAGE
		      : JOptionPane.INFORMATION_MESSAGE));
		if (!answer.isError()) return;
	    }
	}
    }
}


