// $Id: EditBuddiesTree.java,v 1.21 2000/04/14 17:04:48 gwilliam Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.io.IOException;
import java.util.*;
import java.net.*;

import org.mitre.simp.util.*;
import org.mitre.simp.client.model.*;


class EditBuddiesTree 
    extends BuddiesTree
{
    public EditBuddiesTree() {
	super();
	addBuddiesAlways = true;
	setRowHeight(0);
    }

    private boolean editingCell = false;
    private synchronized void setEditingCell(boolean b) { editingCell = b; }
    private synchronized boolean isEditingCell() { return editingCell; }

    public void init(final Client client, ClientModel cm, IMA me, XmlProperties profile) {
	this.client = client;
	this.me = me;
	this.cm = cm;
	this.profile = profile;
	this.buddyList = cm.getBuddyList();

	EditBuddiesTreeCellRenderer renderer = 
	    new EditBuddiesTreeCellRenderer(cm);
	setCellRenderer(renderer);

	setCellEditor(new DefaultTreeCellEditor(this, renderer) {
		public boolean isCellEditable(EventObject event) {
		    boolean b = super.isCellEditable(event);
		    if (b) {
			setEditingCell(true);
		    }
		    return b;
		}
		public boolean stopCellEditing() {
		    boolean b = super.stopCellEditing();
		    if (b) {
			setEditingCell(false);
		    }
		    return b;
		}
		public void cancelCellEditing() {
		    super.cancelCellEditing();
		    setEditingCell(false);
		}
	    });
	setEditable(true);

	getModel().addTreeModelListener(new EditBuddiesTreeModelListener());

	// set up a double-click handler
	addMouseListener(new MouseAdapter() {
	    public void mouseClicked(MouseEvent e) {
		int selRow = getRowForLocation(e.getX(), e.getY());
		TreePath selPath = getPathForLocation(e.getX(), e.getY());
		if(selRow != -1) {
		    if(e.getClickCount() == 1) {
			//mySingleClick(selRow, selPath);
		    }
		    else if(e.getClickCount() == 2) {
			IMA ima = getSelectedBuddy(selPath);
			if (ima != null) {
			    Info buddy = getCm().getInfo(ima);
			    if (buddy == null) {
				JOptionPane.showMessageDialog
				    (client,
				     new String[] {
					 "No information is available for",
					 ima.toString()
				     }, "Buddy Info", 
				     JOptionPane.ERROR_MESSAGE);
				return;
			    }
			    DateTime online = buddy.getOnline();
			    String since = online != null
				? "Online Since: "+online.getFormattedDate()
				: "Online time unknown";
			    String[] info = new String[] {
				"IM Address: "+buddy.getRegarding(),
				"Description: "+buddy.getMessage(),
				"Status: "+buddy.getStatus(),
				since
			    };
			    JOptionPane.showMessageDialog
				(client, info, "Buddy Info", 
				 JOptionPane.INFORMATION_MESSAGE);
			}
		    }
		} 
	    }});
	addKeyListener(new KeyHandler());

	cm.addSimpListener(this);
    }

    private class KeyHandler extends KeyAdapter {

	private BuddyListAction bla = new BuddyListAction();

	public void keyReleased(KeyEvent e) {
	    if (isEditingCell()) return;
	    boolean enter = false;
	    boolean delete = false;
	    final BuddyNode bn = getSelectedBuddyNode();
	    final BuddyGroupNode bgn = getSelectedGroupNode();
	    if (bn == null && bgn == null) 
		// what about group nodes?
		return;
	    int keyCode = e.getKeyCode();
	    switch (keyCode) {
	    case KeyEvent.VK_ENTER:
		enter = true;
		break;
	    case KeyEvent.VK_DELETE:
		delete = true;
		break;
	    default:
		return;
	    }
	    if (bn != null) {
		if (enter) {
		    new Thread() {
			    public void run() {
				client.createMessagePanel
				    (bn.getBuddy());
			    }}.start();
		}
		else if (delete) {
		    new Thread() {
			    public void run() {
				IMA buddy = bn.getBuddy();
				BuddyGroupNode pgn = (BuddyGroupNode)
				    bn.getParent();
				if (buddy == null || pgn == null) 
				    throw new RuntimeException
					("BuddyNode without parent or IMA");
				String group = pgn.getGroupName();
				int choice = JOptionPane.showConfirmDialog
				    (client,
				     new String[] {
					 "Do you really want to remove "+buddy,
					 "from the group "+group+"?"
				     },
				     "Remove Buddy...",
				     JOptionPane.YES_NO_OPTION,
				     JOptionPane.QUESTION_MESSAGE);
				if (choice != JOptionPane.YES_OPTION) return;
				//Answer answer = buddyList.reallyRemoveBuddy(buddy, group);
				final IMA f_buddy = buddy;
				final String f_group = group;
				Answer answer = bla.performBuddyListRequest
				    (new Runnable() {
					    public void run() {
						buddyList.removeBuddy
						    (bla, f_buddy, f_group);
					    }
					});
				if (answer.isError())
				    JOptionPane.showMessageDialog
					(client,
					 new String[] {
					     "The following error occurred while trying to remove",
					     buddy+" from "+group+":",
					     answer.getMessage()
					 },
					 "Error",
					 JOptionPane.ERROR_MESSAGE);
			    }}.start();
		}
	    }
	    else if (bgn != null) {
		if (delete) {
		    new Thread() {
			    public void run() {
				String group = bgn.getGroupName();
				int choice = JOptionPane.showConfirmDialog
				    (client,
				     new String[] {
					 "Are you sure you want to remove the group ",
					 "\""+group+"\" and all the buddies in it?"
				     },
				     "Remove Group",
				     JOptionPane.YES_NO_OPTION,
				     JOptionPane.QUESTION_MESSAGE);
				if (choice != JOptionPane.YES_OPTION) return;
				//Answer answer = buddyList.reallyRemoveGroup(group);
				final String f_group = group;
				Answer answer = bla.performBuddyListRequest
				    (new Runnable() {
					    public void run() {
						buddyList.removeGroup(bla, f_group);
					    }
					});
				if (answer.isError())
				    JOptionPane.showMessageDialog
					(client,
					 new String[] {
					     "The following error occurred while trying to remove",
					     "the group \""+group+"\":",
					     answer.getMessage()
					 },
					 "Error",
					 JOptionPane.ERROR_MESSAGE);
			    }}.start();
		}
	    }
	}
    }

    // SimpListener methods

    public synchronized void buddyInfoChanged(BuddyInfoEvent event) {
	Info info = event.getInfo();
	IMA buddy = info.getRegarding();
	if (buddyGroupsTable.get(buddy) != null)
	    repaint();
    }

    /**
     * Renders cells in the edit buddies tree with the buddy's IMA
     * and description, using the icon to indicate their blocked
     * status.
     */
    private class EditBuddiesTreeCellRenderer
	extends DefaultTreeCellRenderer
    {
	private JLabel dummyLabel = new JLabel(null, null, JLabel.LEFT);

	public EditBuddiesTreeCellRenderer(ClientModel cm) {
	}

	public Component getTreeCellRendererComponent(JTree tree,
						      Object value,
						      boolean sel,
						      boolean expanded,
						      boolean leaf,
						      int row,
						      boolean hasFocus) {
	    super.getTreeCellRendererComponent(tree, value, sel,
					       expanded, leaf, row,
					       hasFocus);
	    if (leaf && value instanceof BuddyNode) {
		BuddyNode n = (BuddyNode) value;
		IMA b = n.getBuddy();

		BooleanAnswer answer = client.isBlocked(n.getBuddy());
		if (answer.isError())
		    setIcon(unknownIcon);
		else if (answer.getValue())
		    setIcon(blockedIcon);
		else
		    setIcon(buddyIcon);
		
		setToolTipText(null); //no tool tip
		Info info = cm.getInfo(b);
		String message = b+((info != null) 
				    ? ": \""+info.getMessage()+"\""
				    : "");
		setText(message);
	    }
	    else if (value instanceof BuddyGroupNode) {
		BuddyGroupNode gn = (BuddyGroupNode) value;
		setText(gn.toStringWithTotalCount());
		setToolTipText(null); //no tool tip
		/* Uncomment this to replace "folder" icons
		   if (expanded) 
		   setIcon(expandedIcon);
		   else
		   setIcon(collapsedIcon);
		*/
	    } else {
		throw new ClassCastException
		    ("Bad node class in EditBuddiesTree! "+value.getClass());
	    }
	    //JLabel l = new JLabel(getText(), getIcon(), JLabel.LEFT);
	    dummyLabel.setText(getText());
	    dummyLabel.setIcon(getIcon());
 	    Dimension d = new Dimension(dummyLabel.getPreferredSize());
	    d.setSize(d.getWidth(), iconDimension.getHeight()*1.25);
  	    setMinimumSize(d);
	    setPreferredSize(d);
  	    setMaximumSize(d);
	    return this;
	}
    }

    /**
     * Checks edits on group and buddy nodes in the tree before passing
     * changes on to server.  If changes fail, the edits are reverted.
     */
    private class EditBuddiesTreeModelListener implements TreeModelListener {
	private BuddyTreeNode getChangedNode(TreeModelEvent e) {
            BuddyTreeNode node;
            node = (BuddyTreeNode)
                     (e.getTreePath().getLastPathComponent());

            try {
                int index = e.getChildIndices()[0];
                node = (BuddyTreeNode)
                       (node.getChildAt(index));
            } catch (NullPointerException npe) { }
	    return node;
	}

	public void treeNodesChanged(TreeModelEvent e) {
	    final BuddyTreeNode node = getChangedNode(e);
	    new Thread() {
		    public void run() {
			if (node instanceof BuddyGroupNode) {
			    BuddyGroupNode n = (BuddyGroupNode) node;
			    final String old_name = (String) 
				n.getSavedUserObject();
			    final String new_name = (String) n.getUserObject();
			    if (old_name.equals(new_name))
				return;
			    //Answer answer = buddyList.reallyRenameGroup(new_name, old_name);
			    Answer answer = 
				bla.performBuddyListRequest(new Runnable() {
				    public void run() {
					buddyList.renameGroup
					    (bla, new_name, old_name);
				    }
				});
			    if (answer.isError()) {
				n.setUserObject(old_name);
				if (client.getClientModel().isDebugging())
				    client.getClientModel().log
					("Rename group error: "+
					 answer.getMessage());
			    }
			}
			else if (node instanceof BuddyNode) {
			    BuddyNode n = (BuddyNode) node;
			    IMA old_ima = (IMA)node.getSavedUserObject();
			    Object new_data = node.getUserObject();
			    if (new_data instanceof String) {
				if (!checkIMA((String)new_data, n)) {
				    n.setUserObject(old_ima);
				}
			    }
			}
			else throw new ClassCastException
			    ("Invalid node type: "+node);
		    }}.start();
        }
        public void treeNodesInserted(TreeModelEvent e) {
        }
        public void treeNodesRemoved(TreeModelEvent e) {
        }
        public void treeStructureChanged(TreeModelEvent e) {
        }	
    }

    private BuddyListAction bla = new BuddyListAction();

    private boolean checkIMA(String buddy, BuddyNode node) {
	IMA bima = null;
	if (client.getClientModel().isDebugging())
	    client.getClientModel().log
		("Editing buddy tree: trying to edit to "+buddy);
	if (buddy != null && !IMA.isIMA(buddy)) {
	    buddy = buddy + "@" + client.getDefaultHost();
	}
	if (IMA.isIMA(buddy)) {
	    try {
		bima = new IMA(buddy);
	    } catch(UnknownHostException uhe) {
		JOptionPane.showMessageDialog
		    (client,
		     new String[] {
			 buddy,
			 "Could not be reached because",
			 "the host "+ uhe.getMessage()
			 +" is unknown.",
			 "Please try again."
		     },
		     "Edit Buddy...", JOptionPane.ERROR_MESSAGE);
		return false;
	    }
	    if (bima.equals(client.getClientModel().getMe())) {
		JOptionPane.showMessageDialog
		    (client, "Cannot be own your buddy.",
		     "Edit Buddy...", JOptionPane.ERROR_MESSAGE);
		return false;
	    }
	    InfoAnswer answer = client.getClientModel().fetch(bima);
	    if (answer.isError()) {
		int choice = JOptionPane.showConfirmDialog
		    (client,
		     new Object[] {
			 "Could not fetch profile information for",
			 new JLabel(buddy.toString(), SwingConstants.CENTER),
			 "due to the following error:",
			 answer.getMessage(),
			 "The user may not exist, or the host may be down.",
			 "Do you wish to add this buddy anyway?",
		     },
		     "Editing Buddy...",
		     JOptionPane.YES_NO_OPTION);
		if (choice != JOptionPane.YES_OPTION)
		    return false;
	    }
	    BuddyGroupNode pnode = (BuddyGroupNode)node.getParent();
	    String group = pnode.getGroupName();
	    IMA obuddy = node.getBuddy();
	    //Answer addans = buddyList.reallyAddBuddy(bima, group);
	    final IMA f_bima = bima;
	    final String f_group = group;
	    Answer addans = bla.performBuddyListRequest(new Runnable() {
		    public void run() {
			buddyList.addBuddy(bla, f_bima, f_group);
		    }
		});
	    if (addans.isError()) {
		JOptionPane.showMessageDialog
		    (client,
		     new Object[] {
			 "Unable to add "+bima,
			 "to your buddies, because of the following error:",
			 addans.getMessage()
		     },
		     "Editing Buddy...",
		     JOptionPane.ERROR_MESSAGE); 
		if (client.getClientModel().isDebugging())
		    client.getClientModel().log
			("Editing buddy tree: failed to add "+bima+
			 ":\n"+addans.getMessage());
		return false;
	    }
	    
	    //Answer remans = buddyList.reallyRemoveBuddy(obuddy, group);
	    final IMA f_obuddy = obuddy;
	    Answer remans = bla.performBuddyListRequest(new Runnable() {
		    public void run() {
			buddyList.removeBuddy(bla, f_obuddy, f_group);
		    }
		});
	    if (remans.isError()) {
		JOptionPane.showMessageDialog
		    (client,
		     new Object[] {
			 "Unable to remove "+obuddy,
			 "from your buddies, because of the following error:",
			 remans.getMessage()
		     },
		     "Editing Buddy...",
		     JOptionPane.ERROR_MESSAGE); 
		if (client.getClientModel().isDebugging())
		    client.getClientModel().log
			("Editing buddy tree: failed to remove "+obuddy
			 +":\n"+remans.getMessage());
		return false;
	    }
	    return true;
	}
	return false;
    }

}
