// $Id: LogFrame.java,v 1.3 2000/03/23 17:01:35 gwilliam Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.IOException;
import java.io.StringWriter;
import java.util.*;

import org.mitre.simp.util.*;
import org.mitre.simp.client.model.*;

class LogFrame extends JFrame
    implements Resourced, ConnectionListener, LogListener
{
    protected static final String resPrefix = "simp.LogWindow.";
    protected static LocalResourcer res = null;
    private static int X_SIZE = -1; 
    private static int Y_SIZE = -1;

    private final MessageAppend textAppend = new MessageAppend();
    private Vector logVector = new Vector();
    private Client client;

    public LogFrame(final Client client) {
	super("Simp Client Log Viewer");
	this.client = client;

	if (res == null)
	    res = new LocalResourcer(resPrefix, Client.resources);
	if (X_SIZE == -1)
	    X_SIZE = res.getResourceInt(WINDOW_X, 300);
	if (Y_SIZE == -1)
	    Y_SIZE = res.getResourceInt(WINDOW_Y, 300);

	JPanel cp = new JPanel();
	cp.setLayout(new BorderLayout());
	//cp.setBorder(BorderFactory.createEtchedBorder());
	//cp.setBorder(BorderFactory.createLoweredBevelBorder());
	cp.setBorder(BorderFactory.createCompoundBorder
		     (BorderFactory.createEtchedBorder(),
		      BorderFactory.createLoweredBevelBorder()));
		     

	JScrollPane scroller
	    = new JScrollPane(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
			      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	JViewport port = scroller.getViewport();
	port.add(textAppend);
  	cp.add(scroller, "Center");

	JMenuBar menuBar = new JMenuBar();
	JMenu file_menu = new JMenu("File");
	file_menu.setMnemonic('F');
	menuBar.add(file_menu);

	JMenuItem hideItem = new JMenuItem("Hide");
	hideItem.setMnemonic('H');
	hideItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    client.showLog(false);
		}});
	file_menu.add(hideItem);
	
	JMenu view_menu = new JMenu("View");
	view_menu.setMnemonic('V');
	menuBar.add(view_menu);
	
	JCheckBoxMenuItem dateItem = 
	    new JCheckBoxMenuItem("Show Message Times", true);
	dateItem.setMnemonic('S');
	dateItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    JCheckBoxMenuItem mi = (JCheckBoxMenuItem) e.getSource();
		    textAppend.setShowDate(mi.getState());
		}});
	view_menu.add(dateItem);

	setJMenuBar(menuBar);

	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		client.showLog(false);
	    }});

	textAppend.setShowDate(true);

	getContentPane().add(cp);
	pack();
	setSize(X_SIZE, Y_SIZE);
	setVisible(false);
    }

    public void messageGenerated(LogEvent e) {
	String message = e.getMessage();
	Throwable t = e.getThrowable();
	add(message);
	if (t != null) {
	    StringWriter writer = new StringWriter();
	    t.printStackTrace(new java.io.PrintWriter(writer));
	    add(writer.toString());
	}
    }

    public void connectionEstablished(ConnectionEvent e) {
    }

    public void connectionClosed(ConnectionEvent e) {
	client.showLog(false);
    }

    private void add(String message) {
	logVector.add(message);
	textAppend.addMessage(message);
    }
}

