// $Id: MessageFrame.java,v 1.34 2000/04/14 17:22:34 gwilliam Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

/**
 * Frame for Instant Messages
 */

package org.mitre.simp.client.released;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.TextAction;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.*;
import java.io.IOException;
import java.net.*;
import org.mitre.simp.util.*;
import org.mitre.simp.launcher.Status;

import org.mitre.simp.client.model.*;


class MessageFrame
    extends JFrame 
    implements Resourced
{

    protected static final String resPrefix = "simp.MessageWindow.";

    protected static LocalResourcer res = null;
    private static int X_SIZE = -1; 
    private static int Y_SIZE = -1;
    
    private MessagePanel mp;
    private Status status;
    private BlockLabel blockLabel;

    private JToolBar toolbar;
    private JPanel mainPanel;

    private boolean frame_is_dead = false;

    private MessageFrame() {
	super();
	this.mp = null;
	this.status = null;
    }

    MessageFrame(final MessagePanel mp, Status status) {
	super("Conversation with "+mp.buddy);
	this.mp = mp;
	this.status = status;
	
	if (res == null)
	    res = new LocalResourcer(resPrefix, Client.resources);
	if (X_SIZE == -1)
	    X_SIZE = res.getResourceInt(WINDOW_X, 300);
	if (Y_SIZE == -1)
	    Y_SIZE = res.getResourceInt(WINDOW_Y, 300);
	    
	
	JPanel cp = new JPanel();
	mainPanel = cp;
	cp.setBorder(BorderFactory.createEtchedBorder());
	cp.setLayout(new BorderLayout());
	cp.add(mp, "Center");

	defaultActions = new Action[] {
	    new CloseAction(),
	    new BuddyInfoAction(),
	    new BuddyBlockUnblockAction(),
	    new BuddyAddAction(),
	    new BuddyBlockAction(),
	    new BuddyRemoveAction(),
	    new BuddyUnblockAction(),
	    //new OptionsAction(),
	    new BeepsAction(),
	    new ShowDatesAction(),
	    new LongNamesAction(),
	    new ToolbarAction(),
	};

	// install the command table
	cmds = new HashMap();
	Action[] acts = getAugmentedActions();
	for (int i = 0; i < acts.length; i++) {
	    Action a = acts[i];
	    cmds.put(a.getValue(Action.NAME), a);
	}

	toolbar = createBars();
	toolbar.setVisible(false);
	mainPanel.add("North",toolbar);

	JPanel statusPanel = new JPanel();
	statusPanel.setLayout(new BorderLayout());
	statusPanel.add(status, BorderLayout.CENTER);
	blockLabel = new BlockLabel();
	blockLabel.addActionListener(getCommandAction(blockunblockAction));
	statusPanel.add(blockLabel, BorderLayout.EAST);
	cp.add(statusPanel, "South");

	getContentPane().add(cp);
	pack();
	setSize(X_SIZE, Y_SIZE);
	show();

	// Whenever the window is activated, we request the focus
	// (which we will pass on to the message panel)
	addWindowListener(new WindowAdapter() {
		public void windowActivated(WindowEvent e) {
		    mp.requestFocus();
		}
		public void windowClosing(WindowEvent e) {
		    mp.closed();
		}
		public void windowClosed(WindowEvent e) {
		    mp.closed();
		}
	    });
	// When we get the focus, we pass it on to the MessagePanel
	addFocusListener(new FocusAdapter() {
		public void focusGained(FocusEvent e) {
		    mp.requestFocus();
		}
	    });

	// When the transcript pane has focus, we
	// disable cut and paste
	mp.messageAppend.addFocusListener(new FocusAdapter() {
	    public void focusGained(FocusEvent e) {
		setActionEnabled(cutAction, false);
		setActionEnabled(pasteAction, false);
	    }});
	// When the input pane has focus, we need
	// to ensure that cut and paste are enabled
	mp.body.addFocusListener(new FocusAdapter() {
	    public void focusGained(FocusEvent e) {
		setActionEnabled(cutAction, true);
		setActionEnabled(pasteAction, true);
	    }});

	String[] inits = {
	    removeAction, blockAction, unblockAction
	};
	for (int i = 0; i < inits.length; i++)
	    ((EnableableAction)getCommandAction(inits[i])).checkEnable();
    }

    // These need to be the same as in the client.properties file
    static final String cutAction = "cut-to-clipboard";
    static final String pasteAction = "paste-from-clipboard";
    private static final String closeAction = "close";
    private static final String optionsAction = "options";
    private static final String infoAction = "info";
    private static final String addremAction = "add/remove";
    private static final String addAction = "add";
    private static final String removeAction = "remove";
    private static final String blockunblockAction = "block/unblock";
    private static final String blockAction = "block";
    private static final String unblockAction = "unblock";
    private static final String beepsAction = "beeps";
    private static final String longnamesAction = "longnames";
    private static final String showdatesAction = "showdates";
    private static final String toolbarAction = "toolbar";

    private Action[] defaultActions;
    private Action[] augmentedActions = null;

    private Action[] getAugmentedActions() {
	if (augmentedActions == null)
	    augmentedActions =
		TextAction.augmentList(mp.body.getActions(), defaultActions);
	return augmentedActions;
    }

    private void setActionEnabled(String actionCommand, boolean enabled) {
	if (getJMenuBar() == null) return;
	JMenuItem mi = getActiveMenuItem(actionCommand);
	if (mi != null) mi.setEnabled(enabled);
	JButton bi = getToolbarButton(actionCommand);
	if (bi != null) bi.setEnabled(enabled);
    }

    JMenuItem getActiveMenuItem(String actionCommand) {
	return (JMenuItem)menu_items.get(getJMenuBar().hashCode()+actionCommand);
    }    

    JButton getToolbarButton(String actionCommand) {
	return (JButton)tool_items.get(actionCommand);
    }
    
    Action getCommandAction(String act) {
	return (Action)cmds.get(act);
    }

    private Map cmds;
    private Map menu_items = Collections.synchronizedMap(new HashMap());
    private Map tool_items = Collections.synchronizedMap(new HashMap());

    // Automagically creates the menu bar and tool bar
    // from the client.properties file, then
    // installs the menu bar and returns the tool bar 
    private JToolBar createBars() {
	JMenuBar bar = new JMenuBar();
	
	String[] menus = res.getResourceTokenized(MENUBAR);
	
	for (int i = 0; i < menus.length; i++) {
	    String[] menu_items = res.getResourceTokenized(menus[i]);
	    JMenu menu = new JMenu(res.getResourceString(menus[i]+labelSuffix));
	    String menu_acc = res.getResourceString(menus[i]+accSuffix);
	    if (menu_acc != null)
		menu.setMnemonic(menu_acc.charAt(0));
	    for (int j = 0; j < menu_items.length; j++) {
		if (menu_items[j].equals("-"))
		    menu.addSeparator();
		String type = res.getResourceString(menu_items[j]+typeSuffix);
		if (type == null || type.equals(TYPE_CHECKBOX)) {
		    JMenuItem mi = null;
		    String label = res.getResourceString(menu_items[j]+labelSuffix);
		    if (type == null)
			mi = new JMenuItem(label);
		    else {
			boolean b = res.getResourceBoolean(menu_items[j]+defaultSuffix);
			mi = new JCheckBoxMenuItem(label, b);
		    }
		    String icon = 
			res.getResourceString(menu_items[j]+iconSuffix);
		    if (icon != null) {
			URL url = MessageFrame.class.getResource(icon);
			if (url != null)
			    mi.setIcon(new ImageIcon(url));
		    }

		    String acc = 
			res.getResourceString(menu_items[j]+accSuffix);
		    if (acc != null)
			mi.setMnemonic(acc.charAt(0));

		    String act = 
			res.getResourceString(menu_items[j]+actionSuffix);
		    if (act == null) act = menu_items[j];
		    mi.setActionCommand(act);
		    Action action = getCommandAction(act);
		    if (action != null) {
			mi.addActionListener(action);
		    } else {
			mi.setEnabled(false);
		    }
		    this.menu_items.put(bar.hashCode()+mi.getActionCommand(), mi);
		    menu.add(mi);
		}
		else if (type.equals(TYPE_SWITCHABLE)) {
		    JMenuItem smi = 
			new SwitchableMenuItem(menu_items[j], res, cmds);
		    this.menu_items.put(bar.hashCode()+smi.getActionCommand(), smi);
		    menu.add(smi);
		}
	    }
	    bar.add(menu);
	}
	
	setJMenuBar(bar);
	

	JToolBar toolbar = new JToolBar();
	String[] tools = res.getResourceTokenized(TOOLBAR);
	if (tools == null) return null;
	for (int i = 0; i < tools.length; i++) {
	    if (tools[i].equals("-"))
		toolbar.addSeparator();
	    else if (tools[i].indexOf("/") != -1) {
		JButton sb = new SwitchableButton(tools[i], res, cmds);
		tool_items.put(sb.getActionCommand(), sb);
		toolbar.add(sb);
	    }
	    else {
		JButton button = new JButton() {
			public float getAlignmentY() { return 0.5f; }
		    };
		button.setRequestFocusEnabled(false);
		button.setMargin(new java.awt.Insets(1,1,1,1));
		
		String icon = res.getResourceString(tools[i]+iconSuffix);
		if (icon != null) {
		    URL url = MessageFrame.class.getResource(icon);
		    if (url != null)
			button.setIcon(new ImageIcon(url));
		}
		String act = res.getResourceString(tools[i]+actionSuffix);
		if (act == null) act = tools[i];
		Action action = getCommandAction(act);
		if (action != null) {
		    button.setActionCommand(act);
		    button.addActionListener(action);
		} else {
		    button.setEnabled(false);
		}

		String tooltip = res.getResourceString(tools[i] + tipSuffix);
		if (tooltip != null)
		    button.setToolTipText(tooltip);
		toolbar.add(button);
		this.tool_items.put(button.getActionCommand(), button);
	    }
	}
	
	return toolbar;
    }

    void setDead() {
	frame_is_dead = true;
    }

    public void dispose() {
	mp.closed();
	super.dispose();
    }

    /*
     * Action classes
     */

    interface EnableableAction {
	public void checkEnable();
    }

    private class CloseAction extends AbstractAction {
	CloseAction() {
	    super(closeAction);
	}
	public void actionPerformed(ActionEvent e) {
	    dispose();
	}
    }

    private class ToolbarAction extends AbstractAction {
	ToolbarAction() {
	    super(toolbarAction);
	}
	public void actionPerformed(ActionEvent e) {
	    JCheckBoxMenuItem mi = (JCheckBoxMenuItem)e.getSource();
	    toolbar.setVisible(mi.getState());
	}
    }

    private class BeepsAction extends AbstractAction {
	BeepsAction() {
	    super(beepsAction);
	}
	public void actionPerformed(ActionEvent e) {
	    JCheckBoxMenuItem mi = (JCheckBoxMenuItem)e.getSource();
	    mp.setSoundEnabled(mi.getState());
	}
    }

    private class LongNamesAction extends AbstractAction {
	LongNamesAction() {
	    super(longnamesAction);
	}
	public void actionPerformed(ActionEvent e) {
	    JCheckBoxMenuItem mi = (JCheckBoxMenuItem)e.getSource();
	    final boolean state = mi.getState();
	    new Thread() {
		    public void run() {
			mp.messageAppend.setShowLongUser(state);
		    }}.start();
	}
    }

    private class ShowDatesAction extends AbstractAction {
	ShowDatesAction() {
	    super(showdatesAction);
	}
	public void actionPerformed(ActionEvent e) {
	    JCheckBoxMenuItem mi = (JCheckBoxMenuItem)e.getSource();
	    final boolean state = mi.getState();
	    new Thread() {
		    public void run() {
			mp.messageAppend.setShowDate(state);
		    }}.start();
	}
    }

    private class BuddyInfoAction extends AbstractAction {
	BuddyInfoAction() {
	    super(infoAction);
	}
	public void actionPerformed(ActionEvent e) {
	    new Thread() {
		public void run() {
		    final String[] info = mp.buddyInfo();
		    if (info == null) {
			return;
		    }
		    SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    JOptionPane.showMessageDialog
				(MessageFrame.this, info,
				 "Info", JOptionPane.INFORMATION_MESSAGE);
			}});
		}}.start();
	}
    }

    private class BuddyBlockUnblockAction extends SwitchableAction 
    {
	BuddyBlockUnblockAction() {
	    super(blockunblockAction);
	}
	public void actionPerformed(final ActionEvent e) {
	    new Thread() {
		public void run() {
		    react(e);
		}}.start();
	}
	private synchronized void react(final ActionEvent e) {
	    BooleanAnswer ba = mp.getClient().isBlocked(mp.buddy);
	    if (!ba.isError()) {
		Answer answer = mp.getClient().setBlocked(mp.buddy,
							  !ba.getValue());
		if (answer.isError()) {
		    final String m = answer.getMessage();
		    SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    JOptionPane.showMessageDialog
				(MessageFrame.this, m, "Error",
				 JOptionPane.ERROR_MESSAGE);
			}});
		}
	    }
	}
    }

    private class BuddyBlockAction extends AbstractAction 
	implements ACLListener, EnableableAction
    {
	ClientModel cm;
	BuddyBlockAction() {
	    super(blockAction);
	    cm = mp.getClient().getClientModel();
	    cm.addACLListener(this);
	}
	public void checkEnable() {
	    BooleanAnswer answer = mp.getClient().isBlocked(mp.buddy);
	    boolean b = (!answer.isError() && answer.getValue());
	    setActionEnabled(blockAction, !b);
	}
	public void aclChanged(ACLChangedEvent e) {
	    if (frame_is_dead) cm.removeACLListener(this);
	    checkEnable();
	}
	public void actionPerformed(final ActionEvent e) {
	    new Thread() {
		public void run() {
		    react(e);
		}}.start();
	}
	private synchronized void react(final ActionEvent e) {
	    BooleanAnswer ba = mp.getClient().isBlocked(mp.buddy);
	    if (!ba.isError() && !ba.getValue()) {
		Answer answer = mp.getClient().setBlocked(mp.buddy, true);
		if (answer.isError()) {
		    final String m = answer.getMessage();
		    SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    JOptionPane.showMessageDialog
				(MessageFrame.this, m, "Error During Block",
				 JOptionPane.ERROR_MESSAGE);
			}});
		}
	    }
	}
    }

    private class BuddyUnblockAction extends AbstractAction
	implements ACLListener, EnableableAction
    {
	ClientModel cm;
	BuddyUnblockAction() {
	    super(unblockAction);
	    cm = mp.getClient().getClientModel();
	    cm.addACLListener(this);
	}
	public void checkEnable() {
	    BooleanAnswer answer = mp.getClient().isBlocked(mp.buddy);
	    boolean b = (!answer.isError() && answer.getValue());
	    setActionEnabled(unblockAction, b);
	}
	public void aclChanged(ACLChangedEvent e) {
	    if (frame_is_dead) cm.removeACLListener(this);
	    checkEnable();
	}
	public void actionPerformed(final ActionEvent e) {
	    new Thread() {
		public void run() {
		    react(e);
		}}.start();
	}
	private synchronized void react(final ActionEvent e) {
	    BooleanAnswer ba = mp.getClient().isBlocked(mp.buddy);
	    if (!ba.isError() && ba.getValue()) {
		Answer answer = mp.getClient().setBlocked(mp.buddy, false);
		if (answer.isError()) {
		    final String m = answer.getMessage();
		    SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    JOptionPane.showMessageDialog
				(MessageFrame.this, m, "Error During Unblock",
				 JOptionPane.ERROR_MESSAGE);
			}});
		}
	    }
	}
    }

    private class BuddyAddAction 
	extends AbstractAction 
	implements Runnable
    {
	BuddyListAction bla = new BuddyListAction();

	BuddyAddAction() {
	    super(addAction);
	}
	public void actionPerformed(ActionEvent e) {
	    new Thread(this).start();
	}
	public void run() {
	    IMA buddy = mp.buddy;
	    Set groups = mp.bl.getGroups(buddy);
	    Set availGroups = mp.bl.getGroups();
	    JComboBox groupBox = new JComboBox(availGroups.toArray());
	    groupBox.setEditable(true);
	    if (groups != null) {
		Iterator it = groups.iterator();
		while (it.hasNext()) {
		    String g = (String)it.next();
		    if (availGroups.contains(g))
			groupBox.removeItem(g);
		}
	    }
	    Object[] message;
	    JList list;
	    JScrollPane scroller;
	    if (groups != null) {		
		scroller = new JScrollPane
		    (ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
		     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JViewport port = scroller.getViewport();
		String[] names = new String[groups.size()];
		groups.toArray(names);
		list = new JList(names);
		list.setSelectionMode
		    (ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		list.setVisibleRowCount(5);
		port.add(list);
		message = new Object[] { 
		    buddy+" currently",
		    "belongs to the following groups:",
		    scroller,
		    "Enter the group to which you wish",
		    "to add "+buddy+":",
		    groupBox,
		    "Or to remove "+buddy,
		    "from a group or groups,",
		    "select the group(s) in the above",
		    "list and click Remove."
		};
		while (true) {
		    int choice = JOptionPane.showOptionDialog
			(MessageFrame.this, message,
			 "Add Buddy...",
			 JOptionPane.DEFAULT_OPTION,
			 JOptionPane.QUESTION_MESSAGE, null,
			 new String[] { "Add", "Remove", "Cancel" },
			 groupBox);
		    switch(choice) {
		    case 0: { // Add
			String group = 
			    ((String)groupBox.getSelectedItem());
			if (group != null) group = group.trim();
			else {
			    JOptionPane.showMessageDialog
				(MessageFrame.this,
				 new String[] {
				     "No group selected!",
				     "Please try again."
				 },
				 "Add Buddy...",
				 JOptionPane.WARNING_MESSAGE);
			    continue;
			}
			if (!mp.bl.groupExists(group)) {
			    choice = JOptionPane.showConfirmDialog
				(MessageFrame.this,
				 new String[] {
				     "The group you specified does not exist.",
				     "Do you wish to create a new group called",
				     "\""+group+"\"?"
				 },
				 "Add Group?",
				 JOptionPane.YES_NO_OPTION);
			    if (choice != JOptionPane.YES_OPTION)
				continue;
			    //Answer addAnswer = mp.bl.reallyAddGroup(group);
			    final String f_group = group;
			    Answer addAnswer = bla.performBuddyListRequest
				(new Runnable() {
					public void run() {
					    mp.bl.addGroup(bla, f_group);
					}
				    });
			    if (addAnswer.isError()) {
				JOptionPane.showMessageDialog
				    (MessageFrame.this,
				     new String[] {
					 "Could not add the group \""+group+"\"",
					 "because of the following error:",
					 addAnswer.getMessage()
				     },
				     "Error", JOptionPane.ERROR_MESSAGE);
				continue;
			    }
			}
			//Answer answer = mp.bl.reallyAddBuddy(buddy, group);
			final IMA f_buddy = buddy;
			final String f_group = group;
			Answer answer = bla.performBuddyListRequest
			    (new Runnable() {
				    public void run() {
					mp.bl.addBuddy(bla, f_buddy, f_group);
				    }
				});
			if (answer.isError()) {
			    JOptionPane.showMessageDialog
				(MessageFrame.this,
				 new Object[] {
				     buddy,
				     "could not be added to the group",
				     "\""+group+"\"",
				     "because of the following error:",
				     "\""+answer.getMessage()+"\"",
				     "Please try again."
				 },
				 "Error", JOptionPane.ERROR_MESSAGE);
			    continue;
			}
			return;
		    }
		    case 1: { // Remove
			Object[] remGroups = list.getSelectedValues();
			list = new JList(remGroups);
			list.setVisibleRowCount(Math.min(5, remGroups.length));
			scroller = new JScrollPane
			    (ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
			     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
			scroller.getViewport().add(list);
			choice = JOptionPane.showConfirmDialog
			    (MessageFrame.this,
			     new Object[] {
				 "Are you sure you want to remove",
				 buddy,
				 "from the following groups?",
				 scroller
			     },
			     "Remove Buddy?",
			     JOptionPane.YES_NO_OPTION);
			if (choice != JOptionPane.YES_OPTION)
			    return;
			HashSet msgs = new HashSet();
			for (int i = 0; i < remGroups.length; i++) {
			    final String group = (String)remGroups[i];
			    //Answer answer = mp.bl.reallyRemoveBuddy(mp.buddy, group);
			    Answer answer = bla.performBuddyListRequest
				(new Runnable() {
				    public void run() {
					mp.bl.removeBuddy(bla, mp.buddy, group);
				    }
				});
			    if (answer.isError()) {
				msgs.add(group+": "+answer.getMessage());
			    }
			}
			if (msgs.size() > 0) {
			    list = new JList(msgs.toArray());
			    scroller = new JScrollPane
				(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
			    scroller.getViewport().add(list);
			    JOptionPane.showMessageDialog
				(MessageFrame.this,
				 new Object[] {
				     "The following errors occurred",
				     "while trying to remove",
				     buddy+":",
				     scroller
				 }, 
				 "Remove Buddy...", 
				 JOptionPane.ERROR_MESSAGE);
			}
			return;
		    }
		    default:
			return;
		    }
		}
	    }
	    else {
		message = new Object[] {
		    buddy,
		    "is currently not in any groups.",
		    "Select a group to which to add",
		    buddy+",",
		    "or enter a new group name:",
		    groupBox
		};
		while (true) {
		    int choice = JOptionPane.showOptionDialog
			(MessageFrame.this, message,
			 "Add Buddy...",
			 JOptionPane.DEFAULT_OPTION,
			 JOptionPane.QUESTION_MESSAGE, null,
			 new String[] { "Add", "Cancel" },
			 groupBox);
		    switch(choice) {
		    case 0: { // Add
			String group = 
			    ((String)groupBox.getSelectedItem()).trim();
			if (!mp.bl.groupExists(group)) {
			    message = new String[] {
				"The group you specified does not exist.",
				"Do you wish to create a new group called",
				"\""+group+"\"?"
			    };
			    choice = JOptionPane.showConfirmDialog
				(MessageFrame.this, message,
				 "Add Group?",
				 JOptionPane.YES_NO_OPTION);
			    if (choice != JOptionPane.YES_OPTION)
				continue;
			    //Answer addAnswer = mp.bl.reallyAddGroup(group);
			    final String f_group = group;
			    Answer addAnswer = bla.performBuddyListRequest
				(new Runnable() {
				    public void run() {
					mp.bl.addGroup(bla, f_group);
				    }
				});
			    if (addAnswer.isError()) {
				JOptionPane.showMessageDialog
				    (MessageFrame.this,
				     new String[] {
					 "Could not add the group \""+group+"\"",
					 "because of the following error:",
					 addAnswer.getMessage()
				     },
				     "Error", JOptionPane.ERROR_MESSAGE);
				continue;
			    }
			}
			//Answer answer = mp.bl.reallyAddBuddy(buddy, group);
			final IMA f_buddy = buddy;
			final String f_group = group;
			Answer answer = bla.performBuddyListRequest
			    (new Runnable() {
				    public void run() {
					mp.bl.addBuddy(bla, f_buddy, f_group);
				    }
				});
			if (answer.isError()) {
			    JOptionPane.showMessageDialog
				(MessageFrame.this,
				 new String[] {
				     buddy + " could not be added to "+group+
				     " because of the following error",
				     answer.getMessage(),
				     "Please try again."
				 },
				 "Error", JOptionPane.ERROR_MESSAGE);
			    continue;
			}
			return;
		    }
		    
		    default:
			return;
		    }
		}
	    }
	}
    }

    private class BuddyRemoveAction extends AbstractAction
	implements BuddyListListener, EnableableAction, Runnable
    {
	BuddyListAction bla = new BuddyListAction();
	BuddyList bl;
	BuddyRemoveAction() {
	    super(removeAction);
	    bl = mp.bl;
	    bl.addBuddyListListener(this);
	}
	public void checkEnable() {
	    Set groups = mp.bl.getGroups(mp.buddy);
	    boolean b = (groups != null && groups.size() > 0);
	    setActionEnabled(removeAction, b);
	}
	public void groupAdded(BuddyListEvent e) {}
	public void groupRemoved(BuddyListEvent e) {}
	public void groupRenamed(GroupRenameEvent e) {}
	public void buddyAdded(GroupEvent e) {
	    if (frame_is_dead) bl.removeBuddyListListener(this);
	    if (!mp.buddy.equals(e.getBuddy())) return;
	    checkEnable();
	}
	public void buddyRemoved(GroupEvent e) {
	    if (frame_is_dead) bl.removeBuddyListListener(this);
	    if (!mp.buddy.equals(e.getBuddy())) return;
	    checkEnable();
	}
	public void actionPerformed(ActionEvent e) {
	    new Thread(this).start();
	}
	public void run() {
	    IMA buddy = mp.buddy;
	    Set groups = bl.getGroups(buddy);
	    Object[] message;
	    JList list;
	    JScrollPane scroller;
	    if (groups != null) {		
		scroller = new JScrollPane
		    (ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
		     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JViewport port = scroller.getViewport();
		String[] names = new String[groups.size()];
		groups.toArray(names);
		list = new JList(names);
		list.setSelectionMode
		    (ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		list.setVisibleRowCount(5);
		port.add(list);
		message = new Object[] { 
		    buddy+" currently",
		    "belongs to the following groups:",
		    scroller,
		    "Select any number of groups from",
		    "which to remove this buddy."
		};
		int choice = JOptionPane.showOptionDialog
		    (MessageFrame.this, message,
		     "Remove Buddy...",
		     JOptionPane.DEFAULT_OPTION,
		     JOptionPane.QUESTION_MESSAGE, null,
		     new String[] { "Remove", "Cancel" },
		     list);
		if (choice != 0) return;
		Object[] remGroups = list.getSelectedValues();
		list = new JList(remGroups);
		list.setVisibleRowCount(Math.min(5, remGroups.length));
		scroller = new JScrollPane
		    (ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
		     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scroller.getViewport().add(list);
		choice = JOptionPane.showConfirmDialog
		    (MessageFrame.this,
		     new Object[] {
			 "Are you sure you want to remove",
			 buddy,
			 "from the following groups?",
			 scroller
		     },
		     "Remove Buddy?",
		     JOptionPane.YES_NO_OPTION);
		if (choice != JOptionPane.YES_OPTION)
		    return;
		HashSet msgs = new HashSet();
		for (int i = 0; i < remGroups.length; i++) {
		    final String group = (String)remGroups[i];
		    //Answer answer = mp.bl.reallyRemoveBuddy(mp.buddy, group);
		    Answer answer = bla.performBuddyListRequest
			(new Runnable() {
				public void run() {
				    mp.bl.removeBuddy(bla, mp.buddy, group);
				}
			    });
		    if (answer.isError()) {
			msgs.add(group+": "+answer.getMessage());
		    }
		}
		if (msgs.size() > 0) {
		    list = new JList(msgs.toArray());
		    scroller = new JScrollPane
			(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
			 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		    scroller.getViewport().add(list);
		    JOptionPane.showMessageDialog
			(MessageFrame.this,
			 new Object[] {
			     "The following errors occurred",
			     "while trying to remove",
			     buddy+":",
			     scroller
			 }, 
			 "Remove Buddy...", 
			 JOptionPane.ERROR_MESSAGE);
		}
	    }	    
	}
    }


    protected static ImageIcon blockedIcon;
    protected static ImageIcon buddyIcon;
    protected static ImageIcon unknownIcon;
    protected static Dimension labelSize = new Dimension(24, 24);

    static {
	try {
	    unknownIcon = new ImageIcon (BuddiesTree.class.getResource ("/resources/images/about.gif"));
	    buddyIcon = new ImageIcon (BuddiesTree.class.getResource ("/resources/images/buddy.gif"));
	    blockedIcon = new ImageIcon (BuddiesTree.class.getResource("/resources/images/block.gif"));
	    labelSize = getMaxIconDimension(new Icon[] {
		blockedIcon, buddyIcon, unknownIcon
	    });
	} catch (Throwable t) {
	    t.printStackTrace();
	}
    }

    public static Dimension getMaxIconDimension(Icon[] icons) {
	int x = 0, y = 0;
	for (int i = 0; i < icons.length; i++) {
	    x = Math.max(x, icons[i].getIconWidth());
	    y = Math.max(y, icons[i].getIconHeight());
	}
	x = x + (x/2);
	y = y + (y/2);
	return new Dimension(x, y);
    }

    private class BlockLabel extends JButton implements ACLListener {
	private final static int PAD = 2;
	private String BLOCK_MSG = "Messages from "+
	    mp.buddy.getUser()+" are currently unblocked. Click to Block";
	private String UNBLOCK_MSG = "Messages from "+
	    mp.buddy.getUser()+" are currently blocked. Click to Unblock";
	IMA buddy;

	BlockLabel() {
	    super();

	    addFocusListener(new FocusAdapter() {
		    public void focusGained(FocusEvent e) {
			mp.requestFocus();
		    }
		});

	    Border inner = BorderFactory.createEmptyBorder(PAD, PAD, PAD, PAD);
	    Border outer = BorderFactory.createBevelBorder(BevelBorder.LOWERED);
	    setBorder(BorderFactory.createCompoundBorder(outer, inner));

	    Client client = mp.getClient();
	    final ClientModel cm = client.getClientModel();
	    this.buddy = mp.getBuddy();
	    BooleanAnswer answer = client.isBlocked(buddy);
	    if (!answer.isError())
		setState(answer.getValue());
	    else setIcon(unknownIcon);
	    cm.addACLListener(this);
	    MessageFrame.this.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent event) {
		    Object object = event.getSource();
		    if (object == MessageFrame.this) {
			cm.removeACLListener(BlockLabel.this);
		    }
		}});
	    Dimension d = labelSize;
	    setMinimumSize(d);
	    setMaximumSize(d);
	    setPreferredSize(d);

	    setHorizontalAlignment(CENTER);
	    setVerticalAlignment(CENTER);
	}
	public void aclChanged(ACLChangedEvent evt) {
	    setState(mp.getClient().isBlocked(buddy, evt.getACL()));
	}
	private synchronized void setState(boolean blocked) {
	    if (blocked) {
		setIcon(blockedIcon);
		setToolTipText(UNBLOCK_MSG);
	    }
	    else {
		setIcon(buddyIcon);
		setToolTipText(BLOCK_MSG);
	    }
	}
    }
}


