// $Id: MessagePanel.java,v 1.31 2000/04/07 19:28:34 gwilliam Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.*;
import java.util.StringTokenizer;
import java.util.Date;
import java.text.DateFormat;
import java.io.IOException;
import java.net.*;

import org.mitre.simp.util.*;
import org.mitre.simp.launcher.Status;
import org.mitre.simp.client.model.*;

/**
 * Panel for Instant Messages
 * @author Galen B. Williamson
 * @version March 2000
 */
class MessagePanel
    extends JPanel
    implements SimpListener, 
	       DocumentListener
{
    final IMA buddy;
    final ClientModel cm;
    final BuddyList bl;
    private final MessageFrame frame;


    final JTextArea body = new JTextArea();
    final Status status = new Status();
    private final Client parent;
    private boolean conversationBuddy = false;
    private boolean soundEnabled = true;
    final MessageAppend messageAppend;

    MessagePanel(ClientModel cm, IMA buddy, Client cl, Font font) {
	//cl.addMessagePanel(buddy, this);
	this.buddy = buddy;
	this.cm = cm;
	this.parent = cl;
	bl = cm.getBuddyList();
	/* Uncomment this to get a nice "To: buddy" label
	JLabel to = new JLabel("To: ");
	JLabel whom = new JLabel(buddy.toString());
	Box toBox = new Box(BoxLayout.X_AXIS);
        toBox.add(to);
	toBox.add(whom);
	add(toBox,"North");
	*/
	final JTextArea body = this.body;
	messageAppend = new MessageAppend(cm.getMe());

	if (font != null) {
	    messageAppend.setFont(font);
	    body.setFont(font);
	}

	//setBorder(BorderFactory.createEtchedBorder());
	setLayout(new VertFillLayout());
	JScrollPane scroller
	    = new JScrollPane(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
			      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	JViewport port = scroller.getViewport();
	port.add(messageAppend);
  	add(scroller, "Center");
	
	body.getDocument().addDocumentListener(this);
	body.setLineWrap(true);
	body.setRows(2);
	JScrollPane bodyScroller = 
	    new JScrollPane(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
			    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	port = bodyScroller.getViewport();
	port.add(body);

	add(bodyScroller, "South");

	cm.addSimpListener(this);
	status.showStatus("Ready");

	frame = new MessageFrame(this, status);

	addFocusListener(new FocusAdapter() {
		public void focusGained(FocusEvent e) {
		    requestFocus();
		}
	    });

	setRequestFocusEnabled(true);
	
	if (bl.getGroups(buddy) == null) {
	    bl.addConversationBuddy(buddy);
	    conversationBuddy = true;
	}

    }

    Client getClient() {
	return parent;
    }

    IMA getBuddy() {
	return buddy;
    }

    public void setSoundEnabled(boolean state) {
	soundEnabled = state;
    }
    
    // Listener Methods

    public void messageReceived(final MessageEvent e) {
	IMA from = e.getFrom();
	if (buddy.equals(from)) {
	    messageAppend.addMessage(buddy, e.getBody());
	    popToFront();
	}
    }

    private String cachedStatus = Protocol.ONLINE;
    private static DateFormat dt = DateFormat.getDateTimeInstance();

    public void infoReceived(InfoEvent e) {
 	Info info = e.getInfo();
	if (!buddy.equals(info.getRegarding())) 
	    return;
	String status = info.getStatus();
	if (cachedStatus.equals(status))
	    return;
	cachedStatus = status;
	messageAppend.addMessage(buddy.getUser() + " went " + 
				 cachedStatus +" at "+
				 dt.format(new Date(System.currentTimeMillis())));
	popToFront();
    }

    public void subscriptionLeased(LeaseEvent e) {
    }

    public void subscriptionLapsed(LeaseEvent e) {
    }

    public void subscribed(SubscriberEvent e) {
    }

    public void unsubscribed(SubscriberEvent e) {
    }

    public void buddyInfoChanged(BuddyInfoEvent e) {
    }

    private void popToFront() {
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    toFront();
		}
	    });

    }

    public void toFront() { 
	if (frame != null) {
	    frame.toFront();
	    if (!body.hasFocus() && !messageAppend.hasFocus()) {
		beep();
	    }
	}
	requestFocus(); 
    }

    public boolean isFocusCycleRoot() { return true; }
    public boolean isFocusTraversable() { return true; }

    public void requestFocus() {
	JMenuItem mi;
	mi = frame.getActiveMenuItem(frame.cutAction);
	mi.setEnabled(true);
	mi = frame.getActiveMenuItem(frame.pasteAction);
	mi.setEnabled(true);
	body.requestFocus();
    }

    /*
     * input area methods
     */

    // DocumentListener methods
    public void changedUpdate(DocumentEvent e) { checkBody(); }
    public void insertUpdate(DocumentEvent e) { checkBody(); }
    public void removeUpdate(DocumentEvent e) { checkBody(); }

    private void checkBody() {
	SwingUtilities.invokeLater
	    (new Runnable() {
		    public void run() {
			String text = body.getText();
			if (text.endsWith("\n")) {
			    final String bod = text.trim();
			    if (bod.equals("")) {
				status.showStatus("Body missing");
				body.setText(null);
				return;
			    }
			    status.showStatus("Sending to " + buddy);
			    body.setText(bod);
			    new Thread() {
				    public void run() {
					send(bod);
				    }
				}.start();
			}
		    }
		});
    }

    public void close() {
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    if (frame != null)
			frame.dispose();
		}
	    });
    }

    public void closed() {
	frame.setDead();
	parent.removeMessagePanel(buddy);
  	cm.removeSimpListener(MessagePanel.this);
	if (conversationBuddy && bl.getGroups(buddy) == null) {
	    Answer answer = bl.removeConversationBuddy(buddy);
	    if (answer.isError()) {
		System.err.println(answer.getMessage());
	    }
	}
    }

    private void send(String body) {
	Answer answer = cm.send(buddy, body);
	SwingUtilities.invokeLater(afterSend(answer, body));
    }

    private Runnable afterSend(final Answer answer, final String bod) {
	return new Runnable() {
		public void run() {
		    if (!answer.isError()) {
			body.setText(null);
			status.showStatus(answer.getMessage());
			messageAppend.addMessage(cm.getMe(), bod);
		    }
		    else {
			status.showStatus(answer.getMessage());
			messageAppend.addMessage("Could not send message: "+
						 answer.getMessage());
		    }
		    requestFocus();
		}
	    };
    }
    
    public String[] buddyInfo() {
	Info info = cm.getInfo(buddy);
	if (info == null) {
	    InfoAnswer answer = cm.fetch(buddy);
	    if (!answer.isError()) {
		info = answer.getInfo();
	    }
	    else return new String[] {
		"IM Address: "+buddy,
		"No further information is available."
	    };
	}
	if (info != null)
	    return new String[] {
		"IM Address: "+	buddy,
		"Description: "+	info.getMessage(),
		"Status: "+		info.getStatus(),
		"Online Since: "+	info.getOnline().getFormattedDate()
	    };
	return null;
    }

    void beep() {
	if (soundEnabled && frame != null) {
	    frame.getToolkit().beep();
	}
    }
}

