// $Id: Resourcer.java,v 1.2 2000/02/21 12:45:49 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import java.net.URL;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

class Resourcer
    implements Resourced
{
    protected String properties;
    protected ResourceBundle resources;
    
    protected Resourcer() {}

    public Resourcer(String properties) 
	throws java.util.MissingResourceException
    {
	this.properties = properties;
	resources = ResourceBundle.getBundle(properties);
    }

    /**
     * Get a URL from a ResourceBundle.
     * @param nm name of key
     * @return resource as a URL or null on error
     */
    public URL getResourceURL(String nm) {
	String s = getResourceString(nm);
	if (s == null) return null;
	try {
	    return new URL(s);
	} catch(java.net.MalformedURLException mue) {
	    return null;
	}
    }

    /**
     * Get a string from a ResourceBundle.
     * @param nm name of key
     * @return resource as a string or null on error
     */
    public String getResourceString(String nm) {
	if (resources == null)
	    return null;
	try {
	    return resources.getString(nm);
	}
	catch (java.util.MissingResourceException mre) {
	    return null;
	}
    }

    /**
     * Get a string as an array of tokens from a ResourceBundle.
     * @param nm name of key
     * @return resource as a string or null on error
     */
    public String[] getResourceTokenized(String nm) {
	String tokens = getResourceString(nm);
	if (nm == null || tokens.length() == 0) return null;
	StringTokenizer t = new StringTokenizer(tokens);
	Vector v = new Vector();
	while (t.hasMoreTokens())
	    v.addElement(t.nextToken());
	String[] tokenized = new String[v.size()];
	return (String[])v.toArray(tokenized);
    }

    /**
     * Get a string as an array of tokens from a ResourceBundle.
     * @param nm name of key
     * @param delim delimiter string to pass on to <code>StringTokenizer</code>
     * @return resource as a string or null on error
     */
    public String[] getResourceTokenized(String nm, String delim) {
	String tokens = getResourceString(nm);
	if (nm == null || tokens.length() == 0) return null;
	StringTokenizer t = new StringTokenizer(tokens, delim);
	Vector v = new Vector();
	while (t.hasMoreTokens())
	    v.addElement(t.nextToken());
	String[] tokenized = new String[v.size()];
	return (String[])v.toArray(tokenized);
    }

    /**
     * Get a string as an array of integers from a ResourceBundle.
     * Uses <code>", \t\r\n"</code> as the delimiter string for
     * <code>StringTokenizer</code> 
     * @param nm name of key
     * @return resource as an array of integers or null on error
     */
    public int[] getResourceIntArray(String nm) {
	return getResourceIntArray(nm, ", \t\r\n");
    }

    /**
     * Get a string as an array of integers from a ResourceBundle.
     * @param nm name of key
     * @param delim delimiter string to pass on to
     * <code>StringTokenizer</code> 
     * @return resource as an array of integers or null on error
     */
    public int[] getResourceIntArray(String nm, String delim) {
	String[] s_vals = getResourceTokenized(nm, delim);
	int[] vals = new int[s_vals.length];
	for (int i = 0; i < vals.length; i++) {
	    try {
		vals[i] = Integer.parseInt(s_vals[i]);
	    } catch (NumberFormatException nfe) {
		return null;
	    }
	}
	return vals;
    }

    /**
     * Get an integer from a ResourceBundle.
     * @param nm name of key
     * @param defaultValue value on error
     * @return resource as an integer or default value on error
     */
    public int getResourceInt(String nm, int defaultValue) {
	String str = getResourceString(nm);
	if (str == null)
	    return defaultValue;
	try {
	    return Integer.parseInt(str);
	}
	catch (NumberFormatException ex) {
	    return defaultValue;
	}
    }


    /**
     * Get a boolean from a ResourceBundle.
     * @param nm name of key
     * @return resource as a boolean or the false value on error
     */
    public boolean getResourceBoolean(String nm) {
	String str = getResourceString(nm);
	return Boolean.valueOf(str).booleanValue();
    }

}
