// $Id: SwitchableButton.java,v 1.3 2000/02/21 12:45:49 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.TextAction;
import java.awt.event.*;
import java.net.URL;
import java.util.*;
import java.io.IOException;
import org.mitre.simp.util.*;

class SwitchableButton
    extends JButton
    implements Switchable, Resourced
{
    int state = 0;
    Icon[] icons = { null, null };
    String[] tips = new String[2];

    SwitchableButton(String res, Resourcer r, Map actions) {
	super();

	setRequestFocusEnabled(false);
	setMargin(new java.awt.Insets(1,1,1,1));

	int slashpos = res.indexOf("/");
	String[] resNames = {
	    res.substring(0, slashpos),
	    res.substring(slashpos+1)
	};
	String def = r.getResourceString(res+defaultSuffix);
	for (int i = 0; i < 2; i++) {
	    String icon = r.getResourceString(resNames[i]+iconSuffix);
	    if (icon != null) {
		URL url = SwitchableButton.class.getResource(icon);
		if (url != null)
		    icons[i] = new ImageIcon(url);
	    }
	    if (resNames[i].equals(def)) {
		state = i;
	    }
	    tips[i] = r.getResourceString(resNames[i] + tipSuffix);
	}
	String act = r.getResourceString(res+actionSuffix);
	if (act == null) act = res;
	setActionCommand(act);
	SwitchableAction action = (SwitchableAction)actions.get(act);
	if (action != null) {
	    addActionListener(action);
	    action.add(this);
	}
	setState();
    }

    private void setState() {
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    setIcon(icons[state]);
		    setToolTipText(tips[state]);
		    invalidate();
		}});
    }

    public void setState(boolean s) {
	state = s ? 1 : 0;
	setState();
    }

    public void toggleState() {
	state = (state == 0) ? 1 : 0;
	setState();
    }

    public boolean getState() {
	return state == 1;
    }

    public float getAlignmentY() { return 0.5f; }
}


