// $Id: SwitchableMenuItem.java,v 1.6 2000/03/13 11:21:30 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.client.released;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.TextAction;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Component;
import java.awt.event.*;
import java.util.*;
import java.io.IOException;
import java.net.*;
import org.mitre.simp.util.*;
import org.mitre.simp.launcher.Status;

class SwitchableMenuItem
    extends JMenuItem
    implements Switchable, Resourced
{
    int state = 0;
    Icon[] icons = { null, null };
    char[] mnemonics = { '\0', '\0' };
    String[] labels = new String[2];

    SwitchableMenuItem(String res, Resourcer r, Map actions) {
	super();
	String def = r.getResourceString(res+defaultSuffix);
	int slashpos = res.indexOf("/");
	String[] resNames = {
	    res.substring(0, slashpos),
	    res.substring(slashpos+1)
	};
	for (int i = 0; i < 2; i++) {
	    labels[i] = r.getResourceString(resNames[i]+labelSuffix);
	    String icon = r.getResourceString(resNames[i]+iconSuffix);
	    if (icon != null) {
		URL url = MessageFrame.class.getResource(icon);
		if (url != null)
		    icons[i] = new ImageIcon(url);
	    }
	    String acc = r.getResourceString(resNames[i]+accSuffix);
	    if (acc != null)
		mnemonics[i] = acc.charAt(0);
	    if (resNames[i].equals(def)) {
		state = i;
	    }
	}
	String act = r.getResourceString(res+actionSuffix);
	if (act == null) act = res;
	setActionCommand(act);
	SwitchableAction action = (SwitchableAction)actions.get(act);
	if (action != null) {
	    addActionListener(action);
	    action.add(this);
	}
	setState(action.getState());
    }

    private void setState() {
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    setText(labels[state]);
		    setIcon(icons[state]);
		    setMnemonic(mnemonics[state]);
		    invalidate();
		}});
    }

    public void setState(boolean s) {
	state = s ? 1 : 0;
	setState();
    }

    public void toggleState() {
	state = (state == 0) ? 1 : 0;
	setState();
    }

    public boolean getState() {
	return state == 1;
    }
}


