// $Id: FixedLengthDate.java,v 1.1 2000/03/13 11:21:31 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.launcher;

import java.util.*;

/** 
 * This class formats dates in a format allowed by the HTTP 1.1
 * protocol described in RFC 2616. The format represents a
 * fixed-length subset of that defined by RFC 1123 (an update to RFC
 * 822).
 * @see java.util.Properties
 * @author John D. Ramsdell
 * @version February 2000 */
public class FixedLengthDate
extends GregorianCalendar
{
    /**
     * Creates a fixed length date formatter.
     */
    public FixedLengthDate() {
	super(TimeZone.getTimeZone("GMT"));
    }

    /**
     * Format a date as a fixed length string.
     */
    public String format() {
	StringBuffer sb = new StringBuffer();
	sb.append(dayName(get(Calendar.DAY_OF_WEEK)));
	sb.append(", ");
	add(sb, get(Calendar.DATE));
	sb.append(' ');
	sb.append(monthName(get(Calendar.MONTH)));
	sb.append(' ');
	sb.append(get(Calendar.YEAR));
	sb.append(' ');
	add(sb, get(Calendar.HOUR_OF_DAY));
	sb.append(':');
	add(sb, get(Calendar.MINUTE));
	sb.append(':');
	add(sb, get(Calendar.SECOND));
	sb.append(" GMT");
	return sb.toString();
    }
		  
    private static String dayName(int day) {
	switch (day) {
	case Calendar.SUNDAY:
	    return "Sun";
	case Calendar.MONDAY:
	    return "Mon";
	case Calendar.TUESDAY:
	    return "Tue";
	case Calendar.WEDNESDAY:
	    return "Wed";
	case Calendar.THURSDAY:
	    return "Thu";
	case Calendar.FRIDAY:
	    return "Fri";
	case Calendar.SATURDAY:
	    return "Sat";
	default:
	    return "";
	}
    }

    private static String monthName(int month) {
	switch (month) {
	case Calendar.JANUARY:
	    return "Jan";
	case Calendar.FEBRUARY:
	    return "Feb";
	case Calendar.MARCH:
	    return "Mar";
	case Calendar.APRIL:
	    return "Apr";
	case Calendar.MAY:
	    return "May";
	case Calendar.JUNE:
	    return "Jun";
	case Calendar.JULY:
	    return "Jul";
	case Calendar.AUGUST:
	    return "Aug";
	case Calendar.SEPTEMBER:
	    return "Sep";
	case Calendar.OCTOBER:
	    return "Oct";
	case Calendar.NOVEMBER:
	    return "Nov";
	case Calendar.DECEMBER:
	    return "Dec";
	default:
	    return "";
	}
    }

    private static void add(StringBuffer sb, int n) {
	if (n < 10) 
	    sb.append('0');
	sb.append(n);
    }

    /**
     * Entry point for testing.
     */
    public static void main(String[] args) {
	FixedLengthDate fld = new FixedLengthDate();
	System.out.println(fld.format());
    }
}
