// $Id: PropEd.java,v 1.3 2000/03/20 12:34:36 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.launcher;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.net.MalformedURLException;

/**
 * A properties editor the Simp Login Client.
 * @version January 2000
 * @author John D. Ramsdell
 * @author Sasha Porto Caskey
 */ 

final class PropEd
    extends JDialog
{
    private final JPanel p = new JPanel();
    private final static int FIELD_SIZE = 30;
    private final JTextField user = new JTextField(FIELD_SIZE);
    private final JTextField host = new JTextField(FIELD_SIZE);
    private final JTextField xpos = new JTextField(FIELD_SIZE);
    private final JTextField ypos = new JTextField(FIELD_SIZE);
    private final JTextField client = new JTextField(FIELD_SIZE);
    private final JTextField textFont = new JTextField(FIELD_SIZE);
    private final JTextField clientHost = new JTextField(FIELD_SIZE);
    private final JTextField clientPort = new JTextField(FIELD_SIZE);
    private final JTextField clientDir = new JTextField(FIELD_SIZE);
    private final JTextField clientLibs = new JTextField(FIELD_SIZE);
    private final Status status = new Status();
    private final static int PAD = 4;

    private final SimpProperties props;
    private final SimpFrame owner;

    PropEd(SimpFrame owner, SimpProperties props) {
	super(owner, "Default Properties Editor");
	this.props = props;
	this.owner = owner;

	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	getContentPane().add(p);
	Border inner = BorderFactory.createEmptyBorder(PAD, PAD, PAD, PAD);
	Border outer = BorderFactory.createEtchedBorder();
	p.setBorder(BorderFactory.createCompoundBorder(outer, inner));
	p.setLayout(new GridBagLayout());

	GridBagUtil gbu = new GridBagUtil(p,PAD);
	int row = 0;
	gbu.addLabel("User logins:", row);
	gbu.addRightComponent(user, row++);
	gbu.addLabel("Default host:", row);
	gbu.addRightComponent(host, row++);
	gbu.addLabel("X position:", row);
	gbu.addRightComponent(xpos, row++);
	gbu.addLabel("Y position:", row);
	gbu.addRightComponent(ypos, row++);
	gbu.addLabel("Client:", row);
	gbu.addRightComponent(client, row++);
	gbu.addLabel("Text Font:", row);
	gbu.addRightComponent(textFont, row++);
	gbu.addLabel("Client host:", row);
	gbu.addRightComponent(clientHost, row++);
	gbu.addLabel("Client port:", row);
	gbu.addRightComponent(clientPort, row++);
	gbu.addLabel("Client directory:", row);
	gbu.addRightComponent(clientDir, row++);
	gbu.addLabel("Client libraries:", row);
	gbu.addRightComponent(clientLibs, row++);
	gbu.addLeftComponent(createButtonPanel(), row++);
	gbu.addStatus(status, row);
	status.showStatus("Editing " + props.getFilename());
	pack();
    }

    private JPanel createButtonPanel() {
	JPanel buttonPanel = new JPanel();
	JButton button;

	button = new JButton("Save");
	button.setMnemonic('S');
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    save();
		    exit();
		}
	    });
	buttonPanel.add(button);

	button = new JButton("Revert");
	button.setMnemonic('R');
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    load();
		}
	    });
	buttonPanel.add(button);

	button = new JButton("Cancel");
	button.setMnemonic('C');
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    exit();
		}
	    });
	buttonPanel.add(button);

	return buttonPanel;
    }

    public void setVisible(boolean b) {
	if (b)
	    load();
	super.setVisible(b);
    }

    private void load() {
	status.showStatus("Editing " + props.getFilename());
	user.setText(props.getProperty(SimpLauncher.SIMP_USERS_KEY, ""));
	host.setText(props.getProperty(SimpLauncher.SIMP_HOST_KEY, ""));
	xpos.setText(props.getProperty(SimpLauncher.SIMP_X_KEY, ""));
	ypos.setText(props.getProperty(SimpLauncher.SIMP_Y_KEY, ""));
	client.setText(props.getProperty(SimpLauncher.SIMP_CLASS_KEY, ""));
	textFont.setText(props.getProperty(SimpLauncher.SIMP_FONT_KEY, ""));
	String path = props.getProperty(SimpLauncher.SIMP_PATH_KEY, "");
	try {
	    URL[] urlpath = Login.getURLs(path);
	    if (urlpath.length > 0) {
		URL first = urlpath[0];
		clientHost.setText(first.getHost());
		int port = first.getPort();
		if (port == -1)
		    clientPort.setText("");
		else
		    clientPort.setText(Integer.toString(port));
		String s = first.getFile();
		clientDir.setText(getDir(s));
		StringBuffer sb = new StringBuffer(getFile(first.getFile()));
		for (int i = 1; i < urlpath.length; i++) {
		    String file = urlpath[i].getFile();
		    sb.append(' ');
		    sb.append(getFile(file));
		}
		clientLibs.setText(sb.toString());
		return;
	    }
	}
	catch (MalformedURLException mue) {
	    status.showStatus("Client info faulty");
	}
	// If ulrpath.length is zero OR exception thrown
	clientHost.setText("");
	clientPort.setText("");
	clientDir.setText("");
	clientLibs.setText("");
    }

    private static String getDir(String path) {
	return path.substring(0, path.lastIndexOf('/') + 1);
    }

    private static String getFile(String path) {
	return path.substring(path.lastIndexOf('/') + 1);
    }

    private String constructPath()
	throws MalformedURLException
    {
	String[] lib = Login.string2array(clientLibs.getText().trim(), ' ');
	if (lib.length == 0)
	    return "";
	String host = clientHost.getText().trim();
	String port = clientPort.getText().trim();
	String dir = clientDir.getText().trim();
	StringBuffer sb = new StringBuffer();
	sb.append(constructURL(host, port, dir, lib[0]));
	for (int i = 1; i < lib.length; i++) {
	    sb.append(';');
	    sb.append(constructURL(host, port, dir, lib[i]));
	}
	return sb.toString();
    }

    private String constructURL(String host, String port, 
				String dir, String file)
	throws MalformedURLException
    {
	StringBuffer sb = new StringBuffer("http://");
	sb.append(host);
	if (!port.equals("")) {
	    sb.append(':');
	    sb.append(port);
	}
	sb.append(dir);
	if (sb.charAt(sb.length() - 1) != '/')
	    sb.append('/');
	sb.append(file);
	URL url = new URL(sb.toString());
	return url.toString();
    }

    private boolean isModified() {
	String userProp = props.getProperty(SimpLauncher.SIMP_USERS_KEY, "");
	String hostProp = props.getProperty(SimpLauncher.SIMP_HOST_KEY, "");
	String xProp = props.getProperty(SimpLauncher.SIMP_X_KEY, "");
	String yProp = props.getProperty(SimpLauncher.SIMP_Y_KEY, "");
	String classProp = props.getProperty(SimpLauncher.SIMP_CLASS_KEY, "");
	String fontProp = props.getProperty(SimpLauncher.SIMP_FONT_KEY, "");
	String pathProp = props.getProperty(SimpLauncher.SIMP_PATH_KEY, "");
	try {
	    return !pathProp.equals(constructPath())
		|| !userProp.equals(user.getText().trim())
		|| !hostProp.equals(host.getText().trim())
		|| !xProp.equals(xpos.getText().trim())
		|| !yProp.equals(ypos.getText().trim())
		|| !classProp.equals(client.getText().trim())
		|| !fontProp.equals(textFont.getText().trim());
	}
	catch (MalformedURLException mue) {
	    status.showStatus("Client info faulty");
	    return true;
	}
    }

    // returns true on error
    private boolean save() {
	String filename = props.getFilename();

	if (!isModified()) {
	    status.showStatus("There are no changes to be saved");
	    return false;
	}
	try {
	    props.setProperty(SimpLauncher.SIMP_PATH_KEY, constructPath());
	}
	catch (MalformedURLException mue) {
	    status.showStatus("Client info faulty--save failed");
	    return true;
	}
	props.setProperty(SimpLauncher.SIMP_USERS_KEY, user.getText().trim());
	props.setProperty(SimpLauncher.SIMP_HOST_KEY, host.getText().trim());
	props.setProperty(SimpLauncher.SIMP_X_KEY, xpos.getText().trim());
	props.setProperty(SimpLauncher.SIMP_Y_KEY, ypos.getText().trim());
	props.setProperty(SimpLauncher.SIMP_CLASS_KEY, client.getText().trim());
	props.setProperty(SimpLauncher.SIMP_FONT_KEY, textFont.getText().trim());
	try {
	    props.store("SIMP Properties");
	}
	catch (FileNotFoundException fnfe) {
	    status.showStatus("Properties file " + filename + 
			      " cannot be opened for output");
	    return true;
	}
	catch (IOException ioe) {
	    status.showStatus("Write to " + filename + " failed");
	    return true;
	}
	owner.reloadProperties();
	status.showStatus("Property file " + filename + " saved");
	return false;
    }

    private void exit() {
	dispose();
    }

    private void queryAndExit() {
	if (isModified()) {
	    String query = 
		"Do you want to save the changes you made to the properties?";
	    int response = JOptionPane.showConfirmDialog(this, query);
	    switch (response) {
	    case JOptionPane.YES_OPTION:
		if (save())	// If an error occurred
		    return;
		else
		    break;
	    case JOptionPane.NO_OPTION:
		break;
	    case JOptionPane.CANCEL_OPTION:
		return;
	    case JOptionPane.CLOSED_OPTION:
		return;
	    }
	}
	exit();
    }
}
