// $Id: SimpLauncher.java,v 1.4 2000/03/31 12:37:14 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.launcher;

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import javax.swing.JOptionPane;

/**
 * Starter application for running a Simp Client.
 * @version March 1997
 * @author John D. Ramsdell
 */ 

public final class SimpLauncher
{
    private SimpLauncher() { }

    private final static String SIMP_PROPERTIES = "simp.properties";
    public final static String SIMP_HOST_KEY = "simp.host";
    final static String SIMP_USERS_KEY = "simp.users";
    final static String SIMP_FONT_KEY = "simp.font";
    final static String SIMP_PATH_KEY = "simp.path";
    final static String SIMP_CLASS_KEY = "simp.class";
    final static String SIMP_X_KEY = "simp.x";
    final static String SIMP_Y_KEY = "simp.y";
    final static String SIMP_HELP_URL_KEY = "simp.help_url";
    public final static String SIMP_ALLOW_MULTIPLE_LOGINS_KEY 
	= "simp.allow_multiple_logins";
    private final static int SIMP_X_DEFAULT = 40;
    private final static int SIMP_Y_DEFAULT = 60;
    // Set version information here now -spc
    private final static int SIMP_MINOR_VERSION = 0;
    private final static int SIMP_MAJOR_VERSION = 1;

    private static String filename; // Available for restart.
    private static SimpProperties props;

    public static SimpProperties getSimpProperties() {
	return props;
    }

    /**
     * Starts a SimpLauncher as an application.
     * If present first arg is the property file.
     */
    public static void main(String args[]) {
	if (args.length > 1) {
	    usage();
	    return;
	}

	filename = args.length == 0 ? SIMP_PROPERTIES : args[0];
	if ("".equals(filename) || '-' == filename.charAt(0)) {
	    usage();
	    return;
	}
	String message = null;
	try {
	    props = new SimpProperties(filename);
	}
	catch (FileNotFoundException fnfe) {
	    message = "Properties file " + filename + " not found";
	}
	catch (IOException ioe) {
	    message = "An error occured while reading property file " + filename;
	}

	if (message != null) {
	    JOptionPane.showMessageDialog(null, message, "Error", 
					  JOptionPane.ERROR_MESSAGE);
	    System.exit(1);
	}

	final File cache = getCacheDir(filename);
	int x = props.getResourceInt(SIMP_X_KEY, SIMP_X_DEFAULT);
	int y = props.getResourceInt(SIMP_Y_KEY, SIMP_Y_DEFAULT);
	final String font = props.getResourceString(SIMP_FONT_KEY);

	try {			// Print forgotten throws here.
	    SimpFrame sf = new SimpFrame("Simp Launcher Version " +
					 getMajorVersion() + "." +
					 getMinorVersion(), props);
	    if (font != null)	// Fonts are broken -- what to do?
		sf.setFont(Font.decode(font));
	    LoginPanel lp = sf.getLoginPanel();
	    lp.addLoginListener(new LoginListener() {
		    public void performLogin(LoginEvent le) {
			String classpath = props.getResourceString(SIMP_PATH_KEY);
			String classname = props.getResourceString(SIMP_CLASS_KEY);
			Login sl = new Login(le.getUser(),
					     le.getPassword(),
					     le.getHost(),
					     font,
					     classpath,
					     classname,
					     cache,
					     (LoginResult)le.getSource());
			new Thread(sl).start();
		    }
		});

	    sf.setLocation(x, y);
	    sf.pack();
	    sf.setVisible(true);
	}
	catch (Throwable t) {
	    System.err.println("uncaught exception: " + t);
	    t.printStackTrace();
	    System.exit(1);
	}
    }
 
    private static void usage() {
	System.err.println("SIMP Launcher Version " + 
			   getMajorVersion() + "." +
			   getMinorVersion());
	System.err.println("Usage: java " + SimpLauncher.class.getName()
			   + " [ PROPERTY FILE ]"); 
	System.err.println("Default property file: " + SIMP_PROPERTIES);
	System.exit(1);
    }

    private static File getCacheDir(String filename) {
	int i = filename.lastIndexOf(File.separatorChar);
	if (i < 0) 
	    return new File(System.getProperty("user.dir"));
	else
	    return new File(filename.substring(0, i));
    }

    public static void restart() {
	if (filename != null)
	    main(new String[] { filename });
    }

    public static int getMajorVersion() {
	return SIMP_MAJOR_VERSION;
    }

    public static int getMinorVersion() {
	return SIMP_MINOR_VERSION;
    }

    public static URL getHelpURL() {
	if (props == null)
	    return null;
	URL url = props.getResourceURL(SIMP_HELP_URL_KEY);
	if (url != null)
	    return url;
	String path = props.getResourceString(SIMP_PATH_KEY);
	String[] lib = Login.string2array(path, ';');
	if (lib == null || lib.length == 0)
	    return null;
	int i = lib[0].lastIndexOf('/');
	if (i < 0)
	    return null;
	String str = lib[0].substring(0, i + 1) + "index.html";
	try {
	    return new URL(str);
	}
	catch (MalformedURLException mue) {
	    return null;
	}
    }
}
