// $Id: ContactPlaceTable.java,v 1.4 2000/02/28 22:25:36 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Maps a user to a contact place.  If the contact place does not
 * exist, and the user has an account, a contact place is created and
 * initialized.  Currently, there is no support for unloading contact
 * places.
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class ContactPlaceTable
{
    private final Hashtable ht = new Hashtable();
    private final SimpService service;

    ContactPlaceTable(SimpService service) {
	this.service = service;
    }

    ContactPlace get(String key) {
	boolean needInit = false;
	synchronized (ht) {
	    ContactPlace place = (ContactPlace)ht.get(key);
	    if (place == null && service.hasAccount(key)) {
		place = new ContactPlace(key, service);
		needInit = true;
		ht.put(key, place);
	    }
	    if (needInit)
		place.init();
	    return place;
	}
    }

    /* There is currently no way to revoke a user account as long as
       the user stays connected. */

    /* Some day, code should be added to unload unused contact places
       from the table.  A contact place without an endpoint and
       subscribers is a candidate for removal.  */

    /*
    ContactPlace remove(String key) {
	return (ContactPlace)ht.remove(key);
    }

    */

    /**
     * destroys all contact places in this table.
     */
    void destroy() {
	synchronized (ht) {
	    Enumeration e = ht.elements();
	    while (e.hasMoreElements()) {
		ContactPlace place = (ContactPlace)e.nextElement();
		place.destroy();
	    }
	}
    }
}
