// $Id: Endpoint.java,v 1.11 2000/04/11 16:56:55 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.util.*;
import java.io.*;
import java.net.*;
import org.mitre.simp.util.*;

/**
 * An endpoint is a contact place's connection with a client.
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class Endpoint
    extends SocketPost		// used to post requests to a client
    implements XpListener
{  
    private final ContactPlace place;
    private final Pool p;
    private final Logger l;

    Endpoint(XpSocket socket, ContactPlace place) 
	throws IOException
    {
	super(socket);
	this.place = place;
	p = place.getSimpService().getPool();
	l = place.getLogger();
    }
 
    public void closed() {
	super.closed();
	place.logout();		// logout when socket closes
    }

    public void xpReceived(XpEvent e) {
	XmlProperties xp = e.getXp();
	String id = xp.getProperty(RESPONSE_KEY);
	if (id != null) {	// relay request responses to the
	    super.xpReceived(e); // socket post method of this name
	    return;
	}
	// respond to requests here
	if (!p.fork(new RequestRunner(place, getXpSocket(), xp, l))) {
	    id = xp.remove(SocketPost.REQUEST_KEY); // if no threads 
	    if (id != null) {	// available, run the request in this one
		XmlProperties resp = place.service(xp);
		resp.setProperty(SocketPost.RESPONSE_KEY, id);
		try {
		    write(resp);
		}
		catch (IOException iox) {
		    try {
			close();
		    }
		    catch (IOException ioe) {
		    }
		    l.log("endpoint: I/O exception during write",
			  iox);
		}
	    }
	}
    }
}
