// $Id: Judge.java,v 1.8 2000/03/10 12:23:36 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.util.*;
import java.io.*;
import java.net.*;
import org.mitre.simp.util.*;

/**
 * A holder for access control lists for the SIMP server.  Instances
 * of this class create and store an encoded form of the ACL that
 * makes the decisions.
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class Judge
{  
    private volatile ACL acl;
    private final Profile profile;

    Judge(String filename, String alternate) {
	profile = new Profile(filename, alternate);
    }

    boolean permit(IMA subject, int operation, boolean validSig) {
	ACL a = acl;
	if (a == null)
	    return false;
	else
	    return a.permit(subject, operation, validSig);
    }

    /**
     * Get the ACL from a the file system if not already done.
     */
    ACL getACL() 
	throws IOException
    {
	if (acl != null)
	    return acl;
	synchronized (profile) {
	    if (acl != null)	// In case another thread was here
		return acl;
	    XmlProperties xp = profile.getProfile();
	    ACL ht = ACL.encode(xp);
	    if (ht == null)
		throw new IOException("bad ACL syntax");
	    acl = ht;
	}
	return acl;
    }

    /**
     * Write ACL to file system and update the acl.
     */
    void setACL(XmlProperties xp) 
	throws IOException
    {
	synchronized (profile) {
	    ACL ht = ACL.encode(xp);
	    if (ht == null)
		throw new IOException("bad ACL syntax");
	    profile.setProfile(xp);
	    acl = ht;
	}
    }
}
