// $Id: Logger.java,v 1.6 2000/03/13 15:37:02 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.io.*;
import java.util.Date;
import org.mitre.simp.util.LogListener;
import org.mitre.simp.util.LogEvent;

/**
 * Logging for the SIMP Server.  An instance of this class is used to
 * write errors and events into a log file.
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class Logger
    implements LogListener
{
    private final SimpProperties props;
    private final boolean verbose;
    private final PrintWriter out;

    /**
     * Creates a logger.
     * @param props used to get a file name and a verbose flag
     */
    Logger(SimpProperties props) { 
	this.props = props;
	String key = SimpProperties.VERBOSE_KEY;
	verbose = props.getPropertyAsBoolean(key);
	key = SimpProperties.LOG_FILE_NAME_KEY;
	String filename = props.getProperty(key);
	Writer w;
	if (filename != null) {
	    try {
		w = new FileWriter(filename, true); // append
	    }
	    catch (IOException iox) {
		w = new OutputStreamWriter(System.out);
	    }
	}
	else			// use System.out by default
	    w = new OutputStreamWriter(System.out);
	out = new PrintWriter(new BufferedWriter(w));
	if (verbose)
	    log("SIMP Server Logger started in verbose mode");
	else
	    log("SIMP Server Logger started");
    }

    /**
     * This method implements the LogListener interface.
     */
    public void messageGenerated(LogEvent e) 
    {
	log(e.getMessage(), e.getThrowable());
    }

    void log(String message) {
	log(message, null);
    }

    /**
     * The main reporting method.
     */
    void log(String message, Throwable t) {
	out.print("[");
	// is this the correct date format?
	out.print(new Date().toString());
	out.print("] ");
	out.println(message);
	if (verbose && t != null)
	    t.printStackTrace(out);
	out.flush();
    }
}
