// $Id: Nonce.java,v 1.4 2000/02/28 22:25:36 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.util.Random;
import java.io.*;
import org.mitre.simp.util.Base64;

/**
 * Creates a random value and encodes it as a string.
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class Nonce
{
    private final static int SIZE = 32;
    private final Random random = new Random();

    /**
     * Returns a random value encoded as a string.
     */
    String nextNonce() {
	ByteArrayOutputStream os = new ByteArrayOutputStream();
	byte[] r = new byte[SIZE];
	random.nextBytes(r);
	os.write(r, 0, r.length);
	os.write(':');
	DataOutputStream dos = new DataOutputStream(os);
	try {
	    dos.writeLong(System.currentTimeMillis());
	    dos.close();
	}
	catch (IOException iox) {
	    // This won't happen with a byte array output stream
	}
	byte[] clear = os.toByteArray();
	char[] nonce = Base64.encode(clear);
	return new String(nonce);
    }

    /**
     * Sets the seed of the random generator.
     */
    void setSeed(long seed) {
	random.setSeed(seed);
    }
}
