// $Id: Password.java,v 1.6 2000/04/11 16:56:55 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.util.*;
import java.io.*;
import org.mitre.simp.util.Protocol;

/**
 * A cache of passwords stored on disk.  The format of the file is a
 * sequence of lines.  Each line gives a user's name, the equals sign
 * (=), and the user's password.
 * @see java.util.Properties 
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class Password
{
    private final Properties passwords = new Properties();
    private long loadTime = -1L;
    private final File passwordFile;

    /**
     * Creates a cache of passwords stored on disk.
     * @param passwordFile the name of the file of passwords
     */
    Password(File passwordFile) {
	this.passwordFile = passwordFile;
    }
    
    /**
     * Returns the password of a user or null if the user does not
     * have an account.  The password file is checked for changes and
     * reload if a change occurred. 
     */
    String getPassword(String user) 
	throws FileNotFoundException, IOException
    {
	if (passwordFile == null || user == null || user.equals("") 
	    || Protocol.SERVER_USER.equals(user))
	    return null;
	long checkTime = passwordFile.lastModified();
	if (checkTime > loadTime) {
	    FileInputStream fis = new FileInputStream(passwordFile);
	    BufferedInputStream bis = new BufferedInputStream(fis);
	    synchronized (passwords) {
		passwords.clear();
		passwords.load(bis);
	    }
	    bis.close();
	    loadTime = checkTime;
	}
	return passwords.getProperty(user);
    }
}
