// $Id: Profile.java,v 1.5 2000/03/10 12:23:37 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.io.*;
import org.mitre.simp.util.*;

/**
 * Profiles for the SIMP server.  Instances of this class
 * create and store profile information.
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class Profile
{  
    private volatile XmlProperties prof;
    private final String filename;
    private final String alternate;

    /**
     * Creates a cache of a profile stored on disk.
     * @param filename the name of the file containing the profile
     * @param alternate the name of the file containing the default profile
     */
    Profile(String filename, String alternate) {
	this.filename = filename;
	this.alternate = alternate;
    }

    /**
     * Get a profile.  Use a cached value if possible.
     */
    XmlProperties getProfile() 
	throws IOException
    {
	if (prof != null)
	    return prof;
	load();
	return prof;
    }

    /**
     * Load a profile from the file system.
     */
    private synchronized void load() 
	throws IOException
    {
	if (prof != null)
	    return;
	XmlProperties profile = new XmlProperties();
	try {
	    FileInputStream p = new FileInputStream(filename);
	    profile.load(new BufferedInputStream(p));
	    p.close();
	}
	catch (FileNotFoundException fnfe) {
	    // Okay, then try the alternate
	    try {
		FileInputStream p = new FileInputStream(alternate);
		profile.load(new BufferedInputStream(p));
		p.close();
	    }
	    catch (FileNotFoundException ex) {
		// A missing file is okay
	    }
	}
	prof = profile;
    }

    /**
     * Store a profile in the file system.
     */
    synchronized void setProfile(XmlProperties profile) 
	throws IOException
    {
	FileOutputStream p = new FileOutputStream(filename);
	profile.store(new BufferedOutputStream(p), null);
	p.close();
	prof = profile;
    }
}
