// $Id: RequestRunner.java,v 1.4 2000/04/11 16:56:56 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.io.IOException;
import java.net.*;
import org.mitre.simp.util.*;

/**
 * Runs a single request via an XmlService object.
 * @see XmlService
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class RequestRunner
	implements Runnable
{
    private final XmlService xs;
    private final XpSocket s;
    private final XmlProperties req;
    private final Logger l;

    /**
     * Creates a request runner.
     * @param xs the XmlService
     * @param s the socket for output
     * @param req the request
     * @param l the logger
     */
    RequestRunner(XmlService xs, XpSocket s, XmlProperties req, Logger l) {
	this.xs = xs;
	this.s = s;
	this.req = req;
	this.l = l;
    }

    public void run() {
	try {
	    String id = req.remove(SocketPost.REQUEST_KEY);
	    if (id == null)	// not a request -- give up
		return;
	    XmlProperties resp = xs.service(req);
	    if (resp != null) {
		resp.setProperty(SocketPost.RESPONSE_KEY, id);
		try {
		    s.write(resp);
		}
		catch (IOException iox) {
		    try {
			s.close();
		    }
		    catch (IOException ioe) {
		    }
		    l.log("request runner: I/O exception during write",
			  iox);
		}
	    }
	}
	catch (Throwable t) {
	    l.log("request runner: internal error", t);
	}
    }
}
