// $Id: ServiceRunner.java,v 1.7 2000/02/28 22:25:37 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.io.InterruptedIOException;
import java.io.IOException;
import java.net.*;
import org.mitre.simp.util.*;

/**
 * Runs a stream runner for each connection accepted.
 * @see XmlService
 * @see StreamRunner
 * @author John D. Ramsdell
 * @version February 2000 
 */
final class ServiceRunner
    implements Runnable
{
    private final static int SO_TIMEOUT = 4 * 60 * 1000;

    private final XmlService xs;
    private final Pool p;
    private final ServerSocket ss;
    private final Logger l;

    /**
     * Creates a service runner.
     * @param xs the XmlService
     * @param p the pool of thread used to service requests
     * @param ss the server socket which accepts connections
     * @param l the logger
     */
    ServiceRunner(XmlService xs, Pool p, ServerSocket ss, Logger l) {
	this.xs = xs;
	this.p = p;
	this.ss = ss;
	this.l = l;
    }

    public void run() {
	try {
	    while (!Thread.interrupted()) {
		Socket s = ss.accept();
		s.setSoTimeout(SO_TIMEOUT);
		StreamRunner sr = new StreamRunner(xs, p, s, l);
		if (!p.fork(sr)) {
		    l.log("service runner: stream closed due to lack of threads.");
		    try {	// Close socket when server is too busy.
			s.close();
		    }
		    catch (IOException ioe) {
		    }
		}
	    }
	    ss.close();
	}
	catch (InterruptedIOException ie) { // Normal exit
	}
	catch (IOException ioe) {
	    l.log("service runner: thread received an I/O exception", ioe);
	}	  
	catch (Throwable t) {
	    l.log("service runner: internal error", t);
	}
	l.log("service runner exiting");
    }
}
