// $Id: SimpProperties.java,v 1.8 2000/02/28 22:25:37 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.util.*;
import java.io.*;

/**
 * Properties for the SIMP Server.
 * @version February 2000
 * @author John D. Ramsdell
 */ 
public class SimpProperties 
{
    // Key names
    final static String LOG_FILE_NAME_KEY 
	= "simp.server.log_file_name";
    final static String PASSWORD_FILE_NAME_KEY 
	= "simp.server.password_file_name";
    final static String ROUTER_POOL_LIMIT_KEY 
	= "simp.server.router_pool_limit";
    final static String NOTIFIER_POOL_LIMIT_KEY 
	= "simp.server.notifier_pool_limit";
    final static String NOTIFIER_PORT_KEY
	= "simp.server.notifier_port";
    final static String COOKIE_TIME_OUT_KEY 
	= "simp.server.cookie_time_out";
    final static String DIGEST_ALGORITHM_KEY 
	= "simp.server.digest_algorithm";
    final static String VERBOSE_KEY 
	= "simp.server.verbose";

    private final Properties props;

    /**
     * Creates a SimpProperties object.
     */
    public SimpProperties() {
	props = new Properties();
    }

    private SimpProperties(Properties props) { // for clone
	this.props = props;
    }

    /**
     * Clones this object.
     */
    public Object clone() {
	return new SimpProperties((Properties)props.clone());
    }

    /**
     * Maps to a java.util.Properties.setProperty().
     * @return String instead of an Object
     */
    public String setProperty(String key, String value) {
	return (String)props.setProperty(key, value);
    }

    /**
     * Maps to a java.util.Properties.getProperty().
     */
    public String getProperty(String key) {
	if (key == null)	// Is this needed?
	    return null;
	return props.getProperty(key);
    }

    /**
     * Maps to a java.util.Properties.getProperty().
     */
    public String getProperty(String key, String defaultValue) {
	if (key == null)	// Is this needed?
	    return defaultValue;
	return props.getProperty(key, defaultValue);
    }

    /**
     * Maps to a java.util.Properties.remove().
     * @return String instead of an Object
     */
    public String remove(String key) {
	return (String)props.remove(key);
    }

    public Enumeration propertyNames() {
	return props.propertyNames();
    }

    public void load(InputStream in)
	throws IOException
    {
	props.load(in);
    }

    public void store(OutputStream out, String header) 
	throws IOException 
    {
	props.store(out, header);
    }

    /**
     * Get an integer.
     * @param key name of key
     * @param defaultValue value on error
     * @return property as an integer or default value on error
     */
    public int getPropertyAsInt(String key, int defaultValue) {
	String str = getProperty(key);
	if (str == null)
	    return defaultValue;
	try {
	    return Integer.parseInt(str);
	}
	catch (NumberFormatException ex) {
	    return defaultValue;
	}
    }

    /**
     * Get a long integer.
     * @param key name of key
     * @param defaultValue value on error
     * @return property as a long integer or default value on error
     */
    public long getPropertyAsLong(String key, long defaultValue) {
	String str = getProperty(key);
	if (str == null)
	    return defaultValue;
	try {
	    return Long.parseLong(str);
	}
	catch (NumberFormatException ex) {
	    return defaultValue;
	}
    }

    /**
     * Get a boolean.
     * @param key name of key
     * @return property as a boolean or the false value on error
     */
    public boolean getPropertyAsBoolean(String key) {
	String str = getProperty(key);
	return Boolean.valueOf(str).booleanValue();
    }
}
