// $Id: StreamRunner.java,v 1.7 2000/02/28 22:25:37 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.io.IOException;
import java.net.Socket;
import org.mitre.simp.util.*;

/**
 * Runs a request runner for each request in a stream associated with
 * a socket.
 * @see XmlService
 * @see RequestRunner
 * @author John D. Ramsdell
 * @version February 2000
 */
final class StreamRunner
    implements Runnable, XpListener
{
    private final XmlService xs;
    private final Pool p;
    private final XpSocket s;
    private final Logger l;

    /**
     * Creates a stream runner.
     * @param xs the XmlService
     * @param p the pool of thread used to service requests
     * @param s the socket of the connection
     * @param l the logger
     */
    StreamRunner(XmlService xs, Pool p, Socket socket, Logger l) 
	throws IOException
    {
	this.xs = xs;
	this.p = p;
	s = new XpSocket(socket);
	this.l = l;
	s.addLogListener(l);
	s.addXpListener(this);
    }

    public void run() {
	s.run();
    }

    public void xpReceived(XpEvent e) {
	XmlProperties req = e.getXp();
	if (!p.fork(new RequestRunner(xs, s, req, l))) {
	    String id = (String)req.remove(SocketPost.REQUEST_KEY);
	    if (id != null) {
		XmlProperties resp = Protocol.error("Server busy");
		resp.setProperty(SocketPost.RESPONSE_KEY, id);
		try {
		    s.write(resp);
		}
		catch (IOException iox) {
		    l.log("stream runner: I/O exception during write",
			  iox);
		}
	    }
	    try {
		s.close();
	    }
	    catch (IOException iox) {
	    }
	}
    }

    public void closed() { }
}
