// $Id: Subscription.java,v 1.5 2000/02/28 22:25:37 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.server;

import java.io.*;
import java.util.*;
import org.mitre.simp.util.*;

/**
 * A subscription associated with a contact place.
 * @version February 2000
 * @author John D. Ramsdell
 */ 
final class Subscription
{
    private final ContactPlace place;
    private final String opaque;
    private final IMA subscriber;
    private final long done;
    private volatile boolean alive = true;

    /**
     * Create a subscription.
     * @param place the contact place associated with the subscription
     * @param opaque the opaque string used in the unsubscribe request
     * @param subscriber the recipient of the presence info
     * @param duration the duration of the subscription
     */
    Subscription(ContactPlace place, String opaque, 
		 IMA subscriber, long duration) {
	this.place = place;
	this.opaque = opaque;
	this.subscriber = subscriber;
	done = duration + System.currentTimeMillis();
    }

    /**
     * Returns the opaque string used in the unsubscribe request.
     */
    String getOpaque() {
	return opaque;
    }

    /**
     * Returns the subscriber.
     */
    IMA getSubscriber() {
	return subscriber;
    }

    /**
     * Returns the time at which this subscription lapses.
     */
    long getDone() {
	return done;
    }

    /**
     * Is this subscription live?
     * @see Queue
     */
    boolean isAlive() {
	return alive;
    }

    /**
     * Set subscription liveliness.
     */
    void setAlive(boolean alive) {
	this.alive = alive;
    }

    /**
     * Report a subscription has lapsed.
     * @see Queue
     */
    void noteSubscriptionLapse() {
	alive = false;
	place.noteSubscriptionLapse(subscriber);
    }
}
