// $Id: Authorization.java,v 1.6 2000/03/01 15:43:14 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

// Authorization using a message digest.

package org.mitre.simp.util;

import java.io.*;
import java.security.*;

/**
 * Creates authorizations used authentication based on HTTP digest
 * authentication.
 * @author John D. Ramsdell
 * @version February 2000
 */
public class Authorization 
{ 
    private final MessageDigest md;

    /**
     * Creates an authorization generator with the specified algorithm
     * for the message digest.  
     */
    public Authorization(String algorithm)
	throws NoSuchAlgorithmException
    {
	md = MessageDigest.getInstance(algorithm);
    }

    /**
     * Returns a message digest formed from the arguments.
     */
    public String getAuthorization(String user, 
				   char[] password, 
				   String nonce)
	throws UnsupportedEncodingException,
	       IOException
    {
	ByteArrayOutputStream os = new ByteArrayOutputStream();
	OutputStreamWriter osw = new OutputStreamWriter(os, "UTF8");
	osw.write(user);
	osw.write(':');
	osw.write(password);
	osw.write(':');
	osw.write(nonce);
	osw.close();
	byte[] buf = os.toByteArray();
	byte[] clear = md.digest(buf);
	md.reset();
	char[] auth = Base64.encode(clear);
	return new String(auth);
    }

    /**
     * Returns the algorithm used to for the message digest.
     */
    public String getAlgorithm() {
	return md.getAlgorithm();
    }
}  
