// $Id: IMA.java,v 1.10 2000/03/01 15:43:15 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

// Instant message addresses.

package org.mitre.simp.util;

import java.util.*;
import java.net.*;

/**
 * Instant message addresses.
 * Instances of this class can be used as keys in hash tables.
 * @author John D. Ramsdell
 * @version February 2000
 */
public class IMA
    implements Comparable
{
    private static InetAddress local;
    static {
	try {
	    local = InetAddress.getLocalHost();
	}
	catch (UnknownHostException ex) {
	}
    }

    private final String user;
    private final InetAddress host;

    /**
     * Creates an instant message address.  If the argument does not
     * contain '@', the local host is assumed.  You can make sure an
     * IMA specifies its host using isIMA. 
     */
    public IMA(String ima) 
	throws UnknownHostException
    {
	if (ima == null)
	    throw new UnknownHostException("null address");
	int i = ima.indexOf('@');
	if (i < 0) {
	    user = ima;
	    host = local;
	}
	else {
	    user = ima.substring(0, i);
	    host = lookupHost(ima.substring(i + 1));
	}
    }

    /**
     * Creates an instant message address.  If the host argument is
     * null, the local host is assumed.  
     */
    public IMA(String user, String host)
	throws UnknownHostException
    {
	this.user = user;
	if (host == null)
	    this.host = local;
	else
	    this.host = lookupHost(host);
	if (user == null)
	    throw new UnknownHostException("null address");
    }

    /**
     * Returns the user part of an instant message address.
     */
    public String getUser() {
	return user;
    }

    /**
     * Returns the host part of an instant message address.
     */
    public String getHost() {
	return host.getHostName();
    }

    /**
     * Returns the host address part of an instant message address.
     */
    public InetAddress getInetAddress() {
	return host;
    }

    /**
     * Is the host part the local machine?
     */
    public boolean isLocal() {
	return host.equals(local);
    }

    /**
     * Is o an IMA that has the same user part and does the host part
     * refer to the a machine with the same IP address?  
     */
    public boolean equals(Object o) {
	if (!(o instanceof IMA))
	    return false;
	IMA ima = (IMA)o;
	return user.equals(ima.user)
	    && host.equals(ima.host);
    }

    public int hashCode() {
	return user.hashCode() + host.hashCode();
    }

    /**
     * Returns the instant message address as a string.
     */
    public String toString() {
	return user + "@" + getHost();
    }

    /**
     * Is ima a string that contains '@'?
     */
    public static boolean isIMA(String ima) {
	return ima != null && ima.indexOf('@') >= 0;
    }

    private final static Hashtable cache = new Hashtable();
    private static InetAddress lookupHost(String hostname)
	throws UnknownHostException
    {
	InetAddress host = (InetAddress)cache.get(hostname);
	if (host != null)
	    return host;
	String ipAddr = InetAddress.getByName(hostname).getHostAddress();
	host = InetAddress.getByName(ipAddr);
	cache.put(hostname, host);
	return host;
    }

    public int compareTo(Object other) {
	IMA o = (IMA)other;
	int n = user.compareTo(o.user);
	if (n == 0)
	    return host.toString().compareTo(o.host.toString());
	return n;
    }
}
