// $Id: Protocol.java,v 1.22 2000/03/20 11:41:28 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

package org.mitre.simp.util;

/**
 * Constants and functions used to implement the SIMP protocol.
 * @author John D. Ramsdell
 * @version February 2000
 */
public class Protocol
{
    private Protocol() { }

    public final static String ACTION = "action";
    public final static String OKAY = "okay";
    public final static String ERROR = "error";
    public final static String MESSAGE = "message";
    public final static String USER = "user";
    public final static String HOST = "host";
    public final static String PORT = "port";
    public final static String LOGIN = "login";
    public final static String CHALLENGE = "challenge";
    public final static String DIVERT = "divert";
    public final static String NONCE = "nonce";
    public final static String OPAQUE = "opaque";
    public final static String ALGORITHM = "algorithm";
    public final static String MIN_VERSION = "min version";
    public final static String MAX_VERSION = "max version";
    public final static String CONNECT = "connect";
    public final static String AUTHORIZATION = "authorization";
    public final static String VERSION = "version";
    public final static String GET_ACL = "get acl";
    public final static String SET_ACL = "set acl";
    public final static String GET_PROFILE = "get profile";
    public final static String SET_PROFILE = "set profile";
    public final static String SEND = "send";
    public final static String FETCH = "fetch";
    public final static String SUBSCRIBE = "subscribe";
    public final static String SUBSCRIBER = "subscriber";
    public final static String UNSUBSCRIBE = "unsubscribe";
    public final static String INQUIRE = "inquire";
    public final static String DURATION = "duration";
    public final static long DURATION_INFINITE = -1;
    public final static String FROM = "from";
    public final static String DATE = "date";
    public final static String REPLY_TO = "reply to";
    public final static String TYPE = "type";
    public final static String TO = "to";
    public final static String BODY = "body";
    public final static String NOTE_CHANGE = "note change";
    public final static String NOTE_SUBSCRIPTION_END
	= "note subscription end";
    public final static String NOTE_SUBSCRIPTION 
	= "note subscription";
    public final static String NOTE_SUBSCRIPTION_LAPSE 
	= "note subscription lapse";
    public final static String DROP_SUBSCRIPTION 
	= "drop subscription";
    public final static String DESCRIBE = "describe";
    public final static String STATUS = "status";
    public final static String OFFLINE = "offline";
    public final static String ONLINE = "online";
    public final static String ON_SINCE = "on since";
    public final static String REGARDING = "regarding";
    public final static String BUDDIES = "buddies";
    public final static String SELF = "self";
    public final static String SERVER_USER = "notifier";

    // ACL constants
    public final static String EVERYBODY = "everybody";
    public final static String CHANGE = "change";
    public final static String END = "end";

    public static boolean isOkay(XmlProperties resp) {
	String r = resp.getProperty(ACTION);
	return OKAY.equals(r);
    }

    public static boolean isError(XmlProperties resp) {
	String r = resp.getProperty(ACTION);
	return ERROR.equals(r);
    }

    public static XmlProperties okay(String message) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, OKAY);
	xp.setProperty(MESSAGE, message);
	return xp;
    }

    public static XmlProperties error(String message) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, ERROR);
	xp.setProperty(MESSAGE, message);
	return xp;
    }

    public static XmlProperties send(IMA to, IMA from, IMA regarding,
				     String type, String body) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, SEND);
	xp.setProperty(TO, to.toString());
	xp.setProperty(FROM, from.toString());
	if (regarding != null)
	    xp.setProperty(REGARDING, regarding.toString());
	xp.setProperty(TYPE, type);
	xp.setProperty(BODY, body);
	return xp;
    }

    public static XmlProperties fetch(IMA to, IMA from) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, FETCH);
	xp.setProperty(TO, to.toString());
	xp.setProperty(FROM, from.toString());
	return xp;
    }

    public static XmlProperties describe(IMA regarding, 
					 DateTime online, 
					 String message) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, DESCRIBE);
	xp.setProperty(REGARDING, regarding.toString());
	setStatus(xp, online, message);
	return xp;
    }

    public static XmlProperties describe(IMA regarding,
					 DateTime online, 
					 String message,
					 long duration,
					 String opaque) {
	XmlProperties xp = describe(regarding, online, message);
	xp.setProperty(DURATION, Long.toString(duration));
	xp.setProperty(OPAQUE, opaque);
	return xp;
    }

    private static void setStatus(XmlProperties xp, 
				  DateTime online,
				  String message) {
	if (online == null) {
	    xp.setProperty(STATUS, Protocol.OFFLINE);
	}
	else {
	    xp.setProperty(STATUS, Protocol.ONLINE);
	    xp.setProperty(ON_SINCE, online.toString());
	}
	xp.setProperty(MESSAGE, message);
    }

    public static XmlProperties subscribe(IMA to, IMA from, long duration) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, SUBSCRIBE);
	xp.setProperty(FROM, from.toString());
	xp.setProperty(TO, to.toString());
	xp.setProperty(DURATION, Long.toString(duration));
	return xp;
    }

    public static XmlProperties unsubscribe(IMA to, IMA from, 
					    String opaque) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, UNSUBSCRIBE);
	xp.setProperty(FROM, from.toString());
	xp.setProperty(TO, to.toString());
	xp.setProperty(OPAQUE, opaque);
	return xp;
    }

     public static XmlProperties inquire(IMA to, IMA from) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, INQUIRE);
	xp.setProperty(FROM, from.toString());
	xp.setProperty(TO, to.toString());
	return xp;
    }

    // Change without the TO field
    public static XmlProperties noteChange(IMA from, 
					   IMA regarding, 
					   DateTime online,
					   String message) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, NOTE_CHANGE);
	xp.setProperty(FROM, from.toString());
	xp.setProperty(REGARDING, regarding.toString());
	setStatus(xp, online, message);
	return xp;
    }

    public static XmlProperties noteSubscriptionEnd(IMA to, IMA from,
						    IMA regarding, 
						    DateTime online,
						    String message) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, NOTE_SUBSCRIPTION_END);
	xp.setProperty(TO, to.toString());
	xp.setProperty(FROM, from.toString());
	xp.setProperty(REGARDING, regarding.toString());
	setStatus(xp, online, message);
	return xp;
    }

    public static XmlProperties noteSubscription(IMA subscriber) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, NOTE_SUBSCRIPTION);
	xp.setProperty(SUBSCRIBER, subscriber.toString());
	return xp;
    }

    public static XmlProperties noteSubscriptionLapse(IMA subscriber) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, NOTE_SUBSCRIPTION_LAPSE);
	xp.setProperty(SUBSCRIBER, subscriber.toString());
	return xp;
    }

    public static XmlProperties divert(String nonce,
				       String opaque,
				       String algorithm,
				       String host,
				       int port) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, DIVERT);
	xp.setProperty(NONCE, nonce);
	xp.setProperty(OPAQUE, opaque);
	xp.setProperty(ALGORITHM, algorithm);
	xp.setProperty(HOST, host);
	xp.setProperty(PORT, Integer.toString(port));
	return xp;
    }

    public static XmlProperties login(String user) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, LOGIN);
	xp.setProperty(USER, user);
	return xp;
    }

    public static XmlProperties challenge(String nonce,
					  String opaque,
					  String algorithm,
					  int minMajor,
					  int minMinor,
					  int maxMajor,
					  int maxMinor,
					  String host,
					  int port) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, CHALLENGE);
	xp.setProperty(NONCE, nonce);
	xp.setProperty(OPAQUE, opaque);
	xp.setProperty(ALGORITHM, algorithm);
	xp.setProperty(MIN_VERSION, minMajor + "." + minMinor);
	xp.setProperty(MAX_VERSION, maxMajor + "." + maxMinor);
	xp.setProperty(HOST, host);
	xp.setProperty(PORT, Integer.toString(port));
	return xp;
    }

    public static XmlProperties connect(String authorization,
					String opaque,
					int major,
					int minor) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, CONNECT);
	xp.setProperty(AUTHORIZATION, authorization);
	xp.setProperty(OPAQUE, opaque);
	xp.setProperty(VERSION, major + "." + minor);
	return xp;
    }

    public static XmlProperties getProfile() {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, GET_PROFILE);
	return xp;
    }

    public static XmlProperties setProfile(XmlProperties prof) {
	XmlProperties xp = (XmlProperties)prof.clone();
	xp.setProperty(ACTION, SET_PROFILE);
	return xp;
    }

    public static XmlProperties getACL() {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, GET_ACL);
	return xp;
    }

    public static XmlProperties setACL(XmlProperties acl) {
	XmlProperties xp = (XmlProperties)acl.clone();
	xp.setProperty(ACTION, SET_ACL);
	return xp;
    }

    public static XmlProperties dropSubscription(IMA subscriber) {
	XmlProperties xp = new XmlProperties();
	xp.setProperty(ACTION, DROP_SUBSCRIPTION);
	xp.setProperty(SUBSCRIBER, subscriber.toString());
	return xp;
    }
}
