// $Id: UTF8XMLWriter.java,v 1.4 2000/02/29 20:30:11 ramsdell Exp $

// Copyright (c) 2000. The MITRE Corporation (http://www.mitre.org/).
// All rights reserved.
// The SIMP Service comes with ABSOLUTELY NO WARRANTY.
// See the SIMP Service License Agreement for details.

// A writer for XML using the UTF8 encoding

package org.mitre.simp.util;

import java.io.*;

/**
 * A writer for XML using the UTF8 encoding.
 * @author John D. Ramsdell
 * @version February 2000
 */
public class UTF8XMLWriter
    extends Writer
{
    private final OutputStreamWriter out;
    
    /**
     * Creates a UTF8 writer from an output stream.
     */
    public UTF8XMLWriter(OutputStream out) 
	throws UnsupportedEncodingException
    {
	this.out = new OutputStreamWriter(out, "UTF8");
    }

    /** 
     * Writes while translating <, >, ', ", and &, into their
     * equivalent entities. 
     */
    public void write(char[] cbuf, int off, int len) 
	throws IOException
    {
	for (int i = off; i < off + len; i++) {
	    char ch = cbuf[i];
	    switch (ch) {
	    case '<':
		out.write("&lt;");
		break;
	    case '>':
		out.write("&gt;");
		break;
	    case '&':
		out.write("&amp;");
		break;
	    case '\'':
		out.write("&apos;");
		break;
	    case '"':
		out.write("&quot;");
		break;
	    default:
		out.write(ch);
	    }
	}
    }

    /** 
     * Writes without translating so that markup can be included in
     * the output. 
     */
    public void markup(String str) 
	throws IOException
    {
	out.write(str);
    }
    
    /**
     * flush output stream.
     */
    public void flush() 
	throws IOException
    {
	out.flush();
    }

    /**
     * close output stream.
     */
    public void close() 
	throws IOException
    {
	out.close();
    }
}
